#!/usr/bin/env python
#-------------------------------------------------------------------------------
# Name:         PIMWrapper.py
#
# Purpose:      This software reads KP-Files.
#
#               This software reads KP-Files containing KP-Index, Ap_Index and F10p7.
#               A format description is given here:
#               ftp://ftp.ngdc.noaa.gov/STP/GEOMAGNETIC_DATA/INDICES/KP_AP/kp_ap.fmt
#
# Needs:        python, f2py, gfortran, PIMWrapper.py, pim.so
#
# Version:      $Id: KPFileReader.py 7470 2013-09-25 12:07:01Z krie_ma $
#               $HeadURL: https://svn.dlr.de/GNSSsoft/trunk/NZ_Projects/IMPC/Tools/Python/PIMPythonInterface/PIM_PythonEnvironment/KPFileReader.py $
#
# Author:       Martin Kriegel
#
# Created:      25.09.2013
# Copyright:    (c) Martin Kriegel 2013
# Mail:         Martin.Kriegel@dlr.de
# Licence:      DLR
#-------------------------------------------------------------------------------

import re
import os
import time
import numpy as np

def main():
    
    kpReaderObj = KPFileReader('./KP')
    
    for hour in range(1,25,3):
        print "Hour: ",hour
        print('KP_Index: ' + str(kpReaderObj.getKPIndex(2011,10,25,hour)))
        
    solarFlux = float(kpReaderObj.getF10p7(2011, 10, 25)[0])

    if np.isnan(solarFlux):
        print "**************** Caution: Solar Flux is NaN! *****************"
    else:
        print "self.solarFlux: ",solarFlux
    

class KPFileReader():
    """
    This class reads KP-files containing KP-indexes and F10p7 values, which are
    necessary for using PIM. You may read the format description of this KP-Files
    at ftp://ftp.ngdc.noaa.gov/STP/GEOMAGNETIC_DATA/INDICES/KP_AP/kp_ap.fmt 
    The data is accessible via the public member function 'getKPIndex' and 'getF10p7'.
    """
    
    def __init__(self, database):
        """
        This is the constructor. The parameter 'database' is the path to the directory
        where all KP-files are stored. (The KP-Files were downloaded during the build 
        of the environment).
        Calling this constructor also initiates the function to parse all the files.
        """
        self.url_base = 'ftp://ftp.ngdc.noaa.gov/STP/GEOMAGNETIC_DATA/INDICES/KP_AP/'
        self.url_format = 'ftp://ftp.ngdc.noaa.gov/STP/GEOMAGNETIC_DATA/INDICES/KP_AP/kp_ap.fmt'
        self.database = database
        self.__readKPFiles()
        pass
        
    def __del__(self):
        """
        This is the constructor of the class.
        """
        pass

    def initData(self):
        """
        This function initializes the data structures to store the parsed values.
        """
        self.KPIndexes = []
        
        KPIndex = {
            'year' : 0,
            'month' : 0,
            'day' : 0,
            'f10p7' : 0,
            'KPindexes' : 0,
            'KPIndexSum' : 0
        }

    def __readKPFiles(self):
        """
        This function parses all the KP-files in the directory specified in the constructor.
        A regular expression is used to fetch all the relevant file according to their
        naming convention (yyyy).
        """
        self.KPIndexes = []
        p = re.compile('[0-9]')
        for kpFile in os.listdir(self.database):
            if p.match(kpFile):
                try:
                    f = open(os.path.join(self.database, kpFile),'r')
                except IOError as inst:
                    print('*** ERROR: Cannot open ' + os.path.join(self.database, kpFile) + ' for writing')
                    print('*** Detailed info: ' + str(inst) + ' at ' + time.asctime() + '\n')
                else:
                    for line in f:          
                        KPIndex = {
                            'year': int(kpFile),
                            'month': int(line[2:4]),
                            'day' : int(line[4:6]),
                            'f10p7' : line[65:70],
                            'f10p7Qualifier' : line[70:71],
                            'KPIndexes': [  int(line[12:14]),
                                            int(line[14:16]),
                                            int(line[16:18]),
                                            int(line[18:20]),
                                            int(line[20:22]),
                                            int(line[22:24]),
                                            int(line[24:26]),
                                            int(line[26:28])],
                            'KPIndexSum' : float(line[28:31])
                        }
                        self.KPIndexes.append(KPIndex)
                    f.close()
                
    def getF10p7(self, year, month, day):
        """
        This function returns the F10p7 value from the KP database for the given time.
        """
        for kpDataSet in self.KPIndexes:
            if kpDataSet['year'] == year and kpDataSet['month'] == month and kpDataSet['day'] == day:
                return [kpDataSet['f10p7'],int(kpDataSet['f10p7Qualifier'])]
        return [0,kpDataSet['f10p7Qualifier']]

        
    def getKPIndex(self, year, month, day, hour):
        """
        This function returns the KP-Index value from the KP database for the given time.
        """
        for kpDataSet in self.KPIndexes:
            if kpDataSet['year'] == year and kpDataSet['month'] == month and kpDataSet['day'] == day:
                if hour  >= 0 and hour < 3:
                    return kpDataSet['KPIndexes'][0]/10.
                elif hour  >= 3 and hour < 6:
                    return kpDataSet['KPIndexes'][1]/10.
                elif hour  >= 6 and hour < 9:
                    return kpDataSet['KPIndexes'][2]/10.
                elif hour  >= 9 and hour < 12:
                    return kpDataSet['KPIndexes'][3]/10.
                elif hour  >= 12 and hour < 15:
                    return kpDataSet['KPIndexes'][4]/10.
                elif hour  >= 15 and hour < 18:
                    return kpDataSet['KPIndexes'][5]/10.
                elif hour  >= 18 and hour < 21:
                    return kpDataSet['KPIndexes'][6]/10.
                elif hour  >= 21 and hour < 24:
                    return kpDataSet['KPIndexes'][7]/10.
                else:
                    return 1.0
                    
if __name__ == "__main__":
    main()

