# coding=utf-8
import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.basemap import Basemap
import matplotlib

class KrigeValidation(object):

    def __init__(self, predictionMatrix, header, path, layer, varmodel, pathCompFile, usedModel):
        '''
        Konstruktor.
        :param predictionMatrix: matrix with the prediction values
        :param header: header for the .dat file
        :param path: Path for the output files
        :param layer: hmF2 / NmF2
        :param varmodel: Variogram model
        :param pathCompFile: Path and filename where the comparison-file is stored
        :param usedModel: NPDM, NPHM or NeQuick
        :return:
        '''
        self.PredictionMatrix = predictionMatrix
        self.ReadMatrix = np.zeros((len(predictionMatrix), len(predictionMatrix[0])))
        self.Header = header
        self.Path = path
        self.Layer = layer
        self.VarModel = varmodel
        self.PathCompFile = pathCompFile
        self.BackModel = usedModel

    def doValidation(self):
        '''
        This function manages the validation procedure
        :return:
        '''
        filename = self.Layer + "_" + self.VarModel + "_" + self.BackModel + "_Validation.dat"
        self.printMatrix(filename)
        self.readMAtrix()
        filename = self.Layer + "_" + self.VarModel + "_" + self.BackModel + "_Diff_Abs_Validation"
        self.printDiffAbs(filename)

        filename = self.Layer + "_" + self.VarModel + "_" + self.BackModel + "_Diff_Rel_Validation"
        self.printDiffRel(filename)

    def readMAtrix(self):
        '''
        This function reads the .dat files. The values are stored in a matrix representing the prediction values on the
        surface. Rows represent lat, columns represent lon
        :return:
        '''
        fobj = open(self.PathCompFile, "r")
        readlines = []
        skipheader = 0
        for line in fobj:
            skipheader += 1                 #skip the header
            if skipheader == 1: continue
            readlines.append(line)
        fobj.close()

        lenRow = len(self.PredictionMatrix) - 1

        row = len(readlines)
        column = len(readlines[0].split())
        if "NmF2" in self.Layer:
            for i in range(0, row):
                splittedRow = readlines[i].split()
                for j in range(0, column):
                    self.ReadMatrix[lenRow - i][j] = float(splittedRow[j]) * (10**12)
        elif "hmF2" in self.Layer:
            for i in range(0, row):
                splittedRow = readlines[i].split()
                for j in range(0, column):
                    self.ReadMatrix[lenRow - i][j] = float(splittedRow[j])
        else:
            raise AssertionError("Layer Error! Layer is neither hmF2 nor NmF2!")

    def printMatrix(self, filename):
        '''
        This function writes the matrix with the prediction values in a file.
        :param filename: Filename
        :return:
        '''
        fobj = open(self.Path + filename, "w")
        fobj.writelines(self.Header + "\n")

        for i in range(len(self.PredictionMatrix)):
            for j in range(len(self.PredictionMatrix)):
                fobj.writelines(str(self.PredictionMatrix[i][j]) + "   ")
            fobj.writelines("\n")
        fobj.close()
        self.writeInListofResults(filename)

    def printDiffAbs(self, filename):
        '''
        This function writes the matrix with the absolute differences between the prediction value and the value in the .dat
        file, in a file.
        :param filename: Filename
        :return:
        '''

        diffAbs = np.abs(np.subtract(self.PredictionMatrix, self.ReadMatrix))
        mean, std = self.calcMeanAndStdMatrix(diffAbs)

        self.plotDiff(diffAbs, filename + ".png")

        addHeader = "\n#Mean Value: " + str(mean) + "\n#Standard deviation: " + str(std)

        fobj = open(self.Path + filename + ".dat", "w")
        fobj.writelines(self.Header + addHeader + "\n")

        for i in range(len(self.PredictionMatrix)):
            for j in range(0, len(self.PredictionMatrix[0])):
                fobj.writelines(str(diffAbs[i][j]) + "   ")
            fobj.writelines("\n")
        fobj.close()
        self.writeInListofResults(filename + ".dat")

    def printDiffRel(self, filename):
        '''
        This function writes the matrix, with the relative differences between the prediction value and the value in the .dat
        file, in a file.
        :param filename:
        :return:
        '''

        diffRel = np.zeros((len(self.PredictionMatrix), len(self.PredictionMatrix[0])))
        for i in range(len(diffRel)):
            for j in range(len(diffRel[0])):
                diffRel[i][j] = (np.abs((self.PredictionMatrix[i][j] - self.ReadMatrix[i][j])) / self.ReadMatrix[i][j]) * 100
                if diffRel[i][j] > 100: diffRel[i][j] = 100

        self.plotDiff(diffRel, filename + ".png")

        mean, std = self.calcMeanAndStdMatrix(diffRel)

        addHeader = "\n#Mean Value: " + str(mean) + "\n#Standard deviation: " + str(std) + "\n"

        fobj = open(self.Path + filename + ".dat", "w")
        fobj.writelines(self.Header + addHeader + "\n")

        for i in range(len(self.PredictionMatrix)):
            for j in range(len(self.PredictionMatrix[0])):
                fobj.writelines(str(diffRel[i][j]) + "   ")
            fobj.writelines("\n")
        fobj.close()
        self.writeInListofResults(filename + ".dat")

    def plotDiff(self, difference, filename):
        plt.figure()
        map = Basemap(projection='cyl', lat_0=0, lon_0=0, resolution='l', ellps="WGS84", area_thresh=1000.0, llcrnrlon=-180,
                      llcrnrlat=-90, urcrnrlon=180, urcrnrlat=90)
        map.drawcoastlines()
        map.drawcountries()
        map.drawmapboundary()
        image = map.imshow(difference, cmap="ocean", origin='upper')
        minVal, maxVal = self.findMinMax(difference)
        colorbarscaling = []
        diff = (maxVal - minVal) / 9
        for i in range(0, 9):
            colorbarscaling.append(minVal + diff * i)
        colorbarscaling.append(maxVal)

        plt.colorbar(image, orientation='horizontal', ticks=np.asarray(colorbarscaling))

        #plot the measurement points
        colorMap = image.get_cmap()
        cNorm = matplotlib.colors.Normalize()
        matplotlib.cm.ScalarMappable(norm=cNorm, cmap=colorMap)
        plt.title(self.Layer + " " + self.BackModel)
        plt.savefig(self.Path + filename)
        plt.close()
        self.writeInListofResults(filename)

    def calcMeanAndStdMatrix(self, matrix):
        '''
        This function calculates the mean and the standard deviation from values in a matrix structure
        :param matrix: NxM Matrix
        :return: Mean and standard deviation
        '''

        summe = 0
        counter = 0
        for i in range(len(matrix)):
            for j in range(len(matrix[0])):
                summe += matrix[i][j]
                counter += 1

        mean = summe / counter

        sum = 0
        for i in range(len(matrix)):
            for j in range(len(matrix[0])):
                sum += (matrix[i][j] - mean)**2

        std = np.sqrt(sum / (counter - 1))

        return mean, std

    def findMinMax(self, matrix):
        '''
        This functions finds the minimum and maximum value in a matrix
        :param matrix: Matrix
        :return: minumum value, maximum value
        '''
        minval = matrix[0][0]
        maxval = matrix[0][0]

        for i in range(len(matrix)):
            for j in range(len(matrix[0])):
                if minval > matrix[i][j]: minval = matrix[i][j]
                elif maxval < matrix[i][j]: maxval = matrix[i][j]
        return minval, maxval

    def writeInListofResults(self, filename):
        '''
        This function will write all filename of the results into one file.
        :param message: message to write
        :return:
        '''
        fobj = open(self.Path + "ListOfResults.dat", "a")
        fobj.writelines(filename + "\n")
        fobj.close()

    def plotHistMeasurementEstimation(self, filename, _xlabel, _ylabel):
        '''
        This function reads the validation file and plots the histogram of the read data (used for residuals).
        :param filename: filename of the validation file incl. path
        :param _xlabel: name of the X-Axis
        :param _ylabel: name of the Y-Axis
        :return:
        '''

        allValues = []
        fobj = open(filename, "r")
        for line in fobj:
            line = line.replace("\n", "")
            lineobj = line.split(";")
            del lineobj[0]
            allValues.extend(lineobj)
        fobj.close()

        plt.figure()

        mean = np.mean(allValues)
        std = np.std(allValues, ddof=1)
        medi = np.median(allValues)
        rms = np.sqrt(np.mean(np.square(allValues)))
        title = "Mean: " + "{:.2f}".format(mean) + " ; Standard Deviation: " + "{:.2f}".format(std) + "\nMedian: " + "{:.2f}".format(medi) + " ; RMS: " + "{:.2f}".format(rms)
        if "Rel" in filename: plt.hist(allValues, range(-100, 100, 5))
        elif "NmF2" in filename: plt.hist(allValues, range(int(-5e11), int(5e11), int(1e10)))
        elif "hmF2" in filename: plt.hist(allValues, range(-100, 100, 10))
        plt.xlabel(_xlabel)
        plt.ylabel(_ylabel)
        plt.title(title)
        newfilename = filename.replace(".dat", ".png")
        plt.savefig(newfilename)
        plt.close()

        #Plot Histogram for every station
        #self.plotHist4RefStations(filename, _xlabel, _ylabel)

    def plotHist4RefStations(self, filename, _xlabel, _ylabel):
        '''
        This function plots the residuals as a histogram for EVERY reference station.
        :param filename: filename (content: residuals)
        :param _xlabel: Name of the x-axis of the histogram plot.
        :param _ylabel: Name of the y-axis of the histogram plot.
        :return:
        '''
        fobj = open(filename, "r")

        for line in fobj:
            line = line.replace("\n", "")
            lineobj = line.split(";")
            refstat = lineobj[0]
            del lineobj[0]
            lineobj = np.asarray(lineobj, dtype=float)
            if len(lineobj) == 0: continue

            plt.figure()
            mean = np.mean(lineobj)
            std = np.std(lineobj, ddof=1)
            medi = np.median(lineobj)
            title = "Mean: " + str(mean) + " ; Standard Deviation: " + str(std) + " ; \nMedian: " + str(medi)
            if "Rel" in filename: plt.hist(lineobj, range(-100, 100, 5))
            elif "NmF2" in filename: plt.hist(lineobj, range(int(-5e11), int(5e11), int(1e10)))
            elif "hmF2" in filename: plt.hist(lineobj, range(-100, 100, 10))
            plt.xlabel(_xlabel)
            plt.ylabel(_ylabel)
            plt.title(title)
            newfilename = filename[:-4] + "_" + refstat + ".png"
            plt.savefig(newfilename)
            plt.close()

        fobj.close()

