import numpy as np
import os
import datetime
import config
import module.merge_data as merge
import Filter.filter_option4 as Filter
import Filter.config as FilterConfig
import MeasurementStations as MS
import KPFileReader.KPFileReader as KPReader
import bin.reconstruct.ModelNPDM_NPHM.NEDMv1 as NEDM
import NeQuick.NeQuick_PythonEnvironment.NeQuickWrapperVectorInput as NeQuick
import OrdinaryCoKriging as myCK

class ManageCoKriging(object):

    def __init__(self):
        '''
        Konstructor. It reads and validates all parameter from the config file. Additionally some parameters are initialized.
        :return:
        '''

        self.Layer = config.layer
        if "hmF2" not in self.Layer and "NmF2" not in self.Layer and "foF2" not in self.Layer:
            self.writeInLog("ERROR: Possible layer input: hmF2, NmF2 and hmF2NmF2 or NmF2hmF2")
            raise AssertionError("Possible layer input: hmF2, NmF2 and hmF2NmF2 or NmF2hmF2")

        self.DoValidation = config.validationfile
        self.Resolution = config.resolution
        self.PathToCompFileNmF2 = config.pathToValidationFileNmF2
        self.PathToCompFilehmF2 = config.pathToValidationFilehmF2
        self.RefStations = config.refStations

        try: bool(self.DoValidation)
        except:
            self.writeInLog("The 'validationfile' variable is not a boolean value!")
            raise AssertionError("The 'validationfile' variable is not a boolean value!")

        try: self.Resolution = float(self.Resolution)
        except:
            self.writeInLog("Resolution is not a number!")
            raise AssertionError("Resolution is not a number!")

        self.Outputfolder = config.outputfolder
        try:
            if not os.path.isdir(self.Outputfolder):
                os.makedirs(self.Outputfolder)
        except:
            self.writeInLog("ERROR: Could not create directory: " + self.Outputfolder)
            raise AssertionError("Could not create directory: " + self.Outputfolder)

        self.OutputfolderLog = config.outputfolderLogs
        try:
            if not os.path.isdir(self.OutputfolderLog):
                os.makedirs(self.OutputfolderLog)
        except:
            self.writeInLog("ERROR: Could not create directory: " + self.OutputfolderLog)
            raise AssertionError("Could not create directory: " + self.OutputfolderLog)


        acdate = datetime.date.today()
        self.LogFileName = "Logfile_" + str(acdate) + ".txt"
        try:
            logfile = open(self.OutputfolderLog + self.LogFileName, "w")
            logfile.close()
        except:
            self.writeInLog("Could not create Log file")
            raise AssertionError("Could not create Log file")

        if self.DoValidation:
            if 180 % self.Resolution != 0:
                self.writeInLog("Resulition is not multiple 180!")
                raise AssertionError("Resulition is not multiple 180!")

            if not self.RefStations:
                self.writeInLog("No reference station mentioned!")
                raise AssertionError("No reference station mentioned!")

        try:
            logfile = open(self.OutputfolderLog + "ListOfResults.dat", "w")
            logfile.close()
        except:
            self.writeInLog("Could not create 'ListOfResults.dat' file")
            raise AssertionError("Could not create 'ListOfResults.dat' file")

        self.writeInLog("---------------------------------------\n---------------------------------------\n" +
                        str(datetime.datetime.now()) + "\nReading config file...\n")

        self.Time = config.time
        try: self.TimeIntervall = int(config.timeintervall)
        except:
            self.writeInLog("ERROR: Timeinterval is not a number!")
            raise AssertionError("Timeinterval is not a number!")

        try: self.Startdoy = int(config.startdoy)
        except:
            self.writeInLog("ERROR: Startdoy is not a number!")
            raise AssertionError("Startdoy is not a number!")

        try: self.Enddoy = int(config.enddoy)
        except:
            self.writeInLog("ERROR: Enddoy is not a number!")
            raise AssertionError("Enddoy is not a number!")


        self.PathSpidr_hmF2 = config.pathspidr_hmF2
        self.PathDataAll_hmF2 = config.pathdata_all_hmF2
        self.PathDidb_hmF2 = config.pathdidb_hmF2
        self.PathNgdc_hmF2 = config.pathngdc_hmF2

        self.PathSpidr_NmF2 = config.pathspidr_NmF2
        self.PathDataAll_NmF2 = config.pathdata_all_NmF2
        self.PathDidb_NmF2 = config.pathdidb_NmF2
        self.PathNgdc_NmF2 = config.pathngdc_NmF2

        try: self.LagDistance = int(config.lagDistance)
        except:
            self.writeInLog("ERROR: Lag distance is not a number!")
            raise AssertionError("Lag distance is not a number!")

        self.PathF107 = config.f107files
        self.VarModel = config.variogrammodel

        if self.LagDistance < 1:
            self.writeInLog("ERROR: Lag distance is < 1")
            raise AssertionError("Lag distance is < 1")

        if not len(self.Time) == 5:
            self.writeInLog("ERROR: Check the format for the time variable (HH:MM)!")
            raise AssertionError("Check the format for the time variable (HH:MM)!")

        try:
            hour = int(self.Time[:-3])
            minutes = int(self.Time[3:])
        except:
            self.writeInLog("ERROR: Could not convert hour and minutes of time in an integer value. Check format (HH:MM)")
            raise AssertionError("Could not convert hour and minutes of time in an integer value. Check format (HH:MM)")

        if not 0 <= hour < 24:
            self.writeInLog("ERROR: The hour of the time is not in the value range (0 <= hour < 24)!")
            raise AssertionError("The hour of the time is not in the value range (0 <= hour < 24)!")
        if not 0 <= minutes < 60:
            self.writeInLog("ERROR: The minute of the time is not in the value range (0 <= hour < 60)!")
            raise AssertionError("The minute of the time is not in the value range (0 <= hour < 60)!")

        self.BackModel = config.backgroundmodel.lower()
        if not "nequick" in self.BackModel and not "nphm" in self.BackModel and not "npdm" in self.BackModel:
            raise AssertionError("The backgroundmodel is neither NeQuick or NPHM nor NPDM!")
        if self.BackModel == "nphm" and "NmF2" in self.Layer:
            raise AssertionError("Wrong model (NPHM) for the layer NmF2!")
        if self.BackModel == "npdm" and "hmF2" in self.Layer:
            raise AssertionError("Wrong model (NPDM) for the layer hmF2!")

        self.Hour = hour + self.convertMinInHour(minutes)

        if self.TimeIntervall < 0:
            self.writeInLog("ERROR: Timeintervall is < 0")
            raise AssertionError("Timeintervall is < 0")

        if not os.path.isdir(self.PathSpidr_hmF2):
            self.writeInLog("ERROR: Could not find directory: " + self.PathSpidr_hmF2)
            raise AssertionError("Could not find directory: " + self.PathSpidr_hmF2)
        if not os.path.isdir(self.PathDataAll_hmF2):
            self.writeInLog("ERROR: Could not find directory: " + self.PathDataAll_hmF2)
            raise AssertionError("Could not find directory: " + self.PathDataAll_hmF2)
        if not os.path.isdir(self.PathDidb_hmF2):
            self.writeInLog("ERROR: Could not find directory: " + self.PathDidb_hmF2)
            raise AssertionError("Could not find directory: " + self.PathDidb_hmF2)
        if not os.path.isdir(self.PathNgdc_hmF2):
            self.writeInLog("ERROR: Could not find directory: " + self.PathNgdc_hmF2)
            raise AssertionError("Could not find directory: " + self.PathNgdc_hmF2)

        if not os.path.isdir(self.PathSpidr_NmF2):
            self.writeInLog("ERROR: Could not find directory: " + self.PathSpidr_NmF2)
            raise AssertionError("Could not find directory: " + self.PathSpidr_NmF2)
        if not os.path.isdir(self.PathDataAll_NmF2):
            self.writeInLog("ERROR: Could not find directory: " + self.PathDataAll_NmF2)
            raise AssertionError("Could not find directory: " + self.PathDataAll_NmF2)
        if not os.path.isdir(self.PathDidb_NmF2):
            self.writeInLog("ERROR: Could not find directory: " + self.PathDidb_NmF2)
            raise AssertionError("Could not find directory: " + self.PathDidb_NmF2)
        if not os.path.isdir(self.PathNgdc_NmF2):
            self.writeInLog("ERROR: Could not find directory: " + self.PathNgdc_NmF2)
            raise AssertionError("Could not find directory: " + self.PathNgdc_NmF2)

        if not config.isotropyAnisotropy == "isotropy" and not config.isotropyAnisotropy == "anisotropy":
            self.writeInLog("ERROR: Choose isotropy or anisotropy")
            raise AssertionError("ERROR: Choose isotropy or anisotropy")

        if not os.path.isdir(self.PathF107):
            self.writeInLog("ERROR: Could not find directory: " + self.PathF107)
            raise AssertionError("Could not find directory: " + self.PathF107)

        if self.Startdoy > self.Enddoy:
            self.writeInLog("ERROR: Startdoy is higher than enddoy!")
            raise AssertionError("Startdoy is higher than enddoy!")

        if self.Startdoy <= 0 or self.Startdoy > 365:
            self.writeInLog("ERROR: Startdoy is not in the intervall 1-365!")
            raise AssertionError("ERROR: Startdoy is not in the intervall 1-365!")

        if self.Enddoy <= 0 or self.Enddoy > 365:
            self.writeInLog("ERROR: Enddoy is not in the intervall 1-365!")
            raise AssertionError("ERROR: Enddoy is not in the intervall 1-365!")

        if "exp" not in self.VarModel and "sph" not in self.VarModel and "gau" not in self.VarModel and "pow" not in self.VarModel:
            self.writeInLog("ERROR: Possibilities for the variogram model are:\n'sph' for spherical\n'exp' for exponential\n'gau' for gaussian\npow for power")
            raise AssertionError("Possibilities for the variogram model are:\n'sph' for spherical\n'exp' for exponential\n'gau' for gaussian\npow for power")

        #initialise some values
        self.AltList = range(1, 600, 1)

        self.F107 = []
        self.Year = 0

        self.writeInLog("Finished reading config file\n")

    def runCoKriging(self):

        if "hmF2" in self.Layer:
            self.PathToCompFile = self.PathToCompFilehmF2
        elif "NmF2" in self.Layer:
            self.PathToCompFile = self.PathToCompFileNmF2
        else:
            self.writeInLog("Layer Error! Layer is neither hmF2 nor NmF2!")
            raise AssertionError("Layer Error! Layer is neither hmF2 nor NmF2!")

        self.StationshmF2 = self.readData("hmF2")
        self.StationsNmF2 = self.readData("foF2")

        self.getf107Values()

        if "nequick" in self.BackModel:
            self.residualsNeQuick()
        elif "npdm" in self.BackModel or "nphm" in self.BackModel:
            self.residualsNPDMNPHM()
        else:
            raise AssertionError("Error with the backgroundmodel in the config file!")

        if "nequick" in self.BackModel:
            if "sph" in self.VarModel:
                self.writeInLog("Start Kriging; NeQuick; spherical variogram model...")
                self.doCoKriging("nequick", variogrammodel="sph")
                self.writeInLog("Finished Kriging; NeQuick; spherical variogram model")

            if "exp" in self.VarModel:
                self.writeInLog("Start Kriging; NeQuick; exponential variogram model...")
                self.doCoKriging("nequick", variogrammodel="exp")
                self.writeInLog("Finished Kriging; NeQuick; exponential variogram model")

            if "gau" in self.VarModel:
                self.writeInLog("Start Kriging; NeQuick; gaussian variogram model...")
                self.doCoKriging("nequick", variogrammodel="gau")
                self.writeInLog("Finished Kriging; NeQuick; gaussian variogram model")

            if "pow" in self.VarModel:
                self.writeInLog("Start Kriging; NeQuick; power variogram model...")
                self.doCoKriging("nequick", variogrammodel="pow")
                self.writeInLog("Finished Kriging; NeQuick; power variogram model")

        ###############################################################
        ################Kriging (Model: NPDM/NPHM)#####################
        ###############################################################
        elif "npdm" in self.BackModel or "nphm" in self.BackModel:
            if "sph" in self.VarModel:
                self.writeInLog("Start Kriging; NPDM; spherical variogram model...")
                self.doCoKriging("npdm", variogrammodel="sph")
                self.writeInLog("Finished Kriging; NPDM; spherical variogram model")

            if "exp" in self.VarModel:
                self.writeInLog("Start Kriging; NPDM; exponential variogram model...")
                self.doCoKriging("npdm", variogrammodel="exp")
                self.writeInLog("Finished Kriging; NPDM; exponential variogram model")

            if "gau" in self.VarModel:
                self.writeInLog("Start Kriging; NPDM; gaussian variogram model...")
                self.doCoKriging("npdm", variogrammodel="gau")
                self.writeInLog("Finished Kriging; NPDM; gaussian variogram model")

            if "pow" in self.VarModel:
                self.writeInLog("Start Kriging; NPDM; power variogram model...")
                self.doCoKriging("npdm", variogrammodel="pow")
                self.writeInLog("Finished Kriging; NPDM; power variogram model")


    def doCoKriging(self, model, variogrammodel):
        '''
        This functions initialized the kriging calculation (inkl. Variograms) and calls the plotting methods. The kriging prediction
        and the variance prediction will be plotted

        :param model: Which model was used for the residuals. In this case NeQuick, NPDM, NPHM. This information is important for
            the filenames
        :param values: "MeasurementStation" - object, with unique stations
        :param variogrammodel: supported Variogrammmodels "exp", "sph" and "gau"
        :return:
        '''

        ordinarykriging = myCK.OrdinaryCoKriging(self.StationshmF2, self.StationsNmF2, variogrammodel, self.Outputfolder, model, self.DoValidation,
                                               self.F107, self.Hour, self.Startdoy, self.Month, self.OutputfolderLog, self.LogFileName, resolution=self.Resolution)
        ordinarykriging.calcVariogramParams(lag=self.LagDistance)
        predval, prederr = ordinarykriging.coKriging()

        #Not needed anymore... but it may be helpful in the future. Comparison between two estimations. Use carefully
        if self.DoValidation:
            if not self.PathToCompFileNmF2 == "" and self.Layer == "NmF2":
                if not os.path.isfile(self.PathToCompFileNmF2):
                    self.writeInLog("Could not find the compare file (NmF2)! Comparison will be skipped.")
                    print "Could not find the compare file (NmF2)! Comparison will be skipped."
                else:
                    self.writeInLog("Started validation for: " + self.Layer + "; " + variogrammodel + "; " + self.BackModel + "...")
                    addHeader = "\n#Unit: electron*meter^(-3)"
                    header = "#Layer: " + self.Layer + "\n#Variogrammodel: " + variogrammodel + "\n#Lag: " + str(self.LagDistance) \
                             + "\n#Resolution: " + str(self.Resolution) + " Grad" + addHeader
                    dovalidation = KrigeVal.KrigeValidation(predval, header, self.Outputfolder, self.Layer, variogrammodel,
                                                            self.PathToCompFile, model)
                    dovalidation.doValidation()

            if not self.PathToCompFilehmF2 == "" and self.Layer == "hmF2":
                if not os.path.isfile(self.PathToCompFilehmF2):
                    self.writeInLog("Could not find the compare file (hmF2)! Comparison will be skipped.")
                    print "Could not find the compare file (hmF2)! Comparison will be skipped."
                else:
                    self.writeInLog("Started validation for: " + self.Layer + "; " + variogrammodel + "; " + self.BackModel + "...")
                    addHeader = "\n#Unit: km"
                    header = "#Layer: " + self.Layer + "\n#Variogrammodel: " + variogrammodel + "\n#Lag: " + str(self.LagDistance) \
                             + "\n#Resolution: " + str(self.Resolution) + " Grad" + addHeader
                    dovalidation = KrigeVal.KrigeValidation(predval, header, self.Outputfolder, self.Layer, variogrammodel,
                                                            self.PathToCompFile, model)
                    dovalidation.doValidation()
                    self.writeInLog("Finished validation for: " + self.Layer + "; " + variogrammodel + "; " + self.BackModel)


        ordinarykriging.plotKrigePrediction(predval, self.Layer + "_Kriging prediction_" + model + " " + variogrammodel)
        #self.writeInListofResults(self.Layer + "_Kriging prediction_" + model + " " + variogrammodel + ".png")
        ordinarykriging.plotPredError(prederr, self.Layer + "_Variance Prediction_" + model + " " + variogrammodel)
        #self.writeInListofResults(self.Layer + "_Kriging predictionError_" + model + " " + variogrammodel + ".png")

    def readData(self, layer):
        '''
        This functions reads the hmF2 and NmF2 data. The Values and the coordinates will be stored in a "MeasurementStation" - object.
        :return: List of all stations (datastructure: MeasurementStations)
        '''

        uniqueStations = MS.Measurementstations(False, layer, refstations=self.RefStations)
        allStations = MS.Measurementstations(True, layer, refstations=self.RefStations)

        merge.param = layer
        if layer == "foF2":
            merge.dlpath_spidr = self.PathSpidr_NmF2
            merge.dlpath_all = self.PathDataAll_NmF2
            merge.dlpath_didb = self.PathDidb_NmF2
            merge.dlpath_ngdc = self.PathNgdc_NmF2
        elif layer == "hmF2":
            merge.dlpath_spidr = self.PathSpidr_hmF2
            merge.dlpath_all = self.PathDataAll_hmF2
            merge.dlpath_didb = self.PathDidb_hmF2
            merge.dlpath_ngdc = self.PathNgdc_hmF2
        else:
            self.writeInLog("ERROR: Could not find Layer: " + layer)
            raise AssertionError("Bad layer input!")

        #merge.merge()
        allData = merge.return_dict()
        if not allData:
            self.writeInLog("ERROR: No Data found for " + layer)
            raise AssertionError("No Data found for " + layer)

        if layer == "foF2": layer = "NmF2"      #filter requirement, it wont accept fof2 as a layer

        if "NmF2" in layer:
            for station in allData.keys():
                values = allData[station]
                for dat in allData[station].keys():
                    if isinstance(values[dat], float):
                        values[dat] = 1.24e10*(values[dat])**2*1e-12

        #Filter
        FilterConfig.doy_start = 8
        FilterConfig.doy_end = 32
        if "hmF2" in layer: FilterConfig.path_input = self.PathDataAll_hmF2
        else: FilterConfig.path_input = self.PathDataAll_NmF2
        FilterConfig.year_start = 2011
        lowerBound, upperBound = Filter.FilterOption4().filter4(layer, allData)

        stations = allData.keys()
        self.Year = int(allData[stations[0]].keys()[0][:4])
        startdom, startmonth = self.getDayOfMonthandMonth(self.Startdoy)

        self.Month = startmonth

        miss_station = []

        for station in stations:
            values = allData[station]
            dates = values.keys()           #Get Date+Time
            #Some stations have no "Meta" Information
            if not allData[station]["Meta"]:
                miss_station.append(station)
                continue
            isStation = False
            for date in dates:
                if date == "Meta": continue
                if len(date) != 16:
                    #print "WARNING: Datum: '" + date + "' is not the excpected dateformat (YYYY-MM-DD HH:MM)"
                    #self.writeInLog("WARNING: Datum: '" + date + "' is not the excpected dateformat (YYYY-MM-DD HH:MM)")
                    continue
                if "NmF2" in self.Layer: values[date] *= 1e12
                doy = self.getDayOfYear(int(date[8:-6]), int(date[5:-9]), int(date[:-12]))
                hour = int(date[11:-3]) + self.convertMinInHour(int(date[14:]))            #get the hour out of the datestring

                moy = doy * 24 * 60 + hour * 60
                minValue, maxValue = self.getMinMaxBoundary(lowerBound, upperBound, moy, station)

                localtimeMin = self.Hour - (self.convertMinInHour(self.TimeIntervall / 2))
                localtimeMax = self.Hour + (self.convertMinInHour(self.TimeIntervall / 2))
                ''' only neccessary, if the day changes in the timerange
                if localtimeMin < 0 and localtimeMax < 0:
                    localtimeMin += 24
                    localtimeMax += 24
                    doy -= 1
                elif localtimeMin >= 24 and localtimeMax >= 24:
                    localtimeMin -= 24
                    localtimeMax -= 24
                    doy += 1
                elif localtimeMin < 0:
                    localtimeMin += 24
                    localtimeMax = 24
                    doy -= 1
                    if localtimeMin <= hour < localtimeMax and self.Startdoy <= doy <= self.Enddoy:

                        if(minValue < values[date] < maxValue):
                            isStation = True
                            self.AllStations.addStation(allData[station]["Meta"]["Lat"], allData[station]["Meta"]["Lon"], values[date], station)
                    continue
                elif localtimeMax >= 24:
                    localtimeMax -= 24
                    localtimeMin = 0
                    doy += 1
                    if localtimeMin <= hour < localtimeMax and self.Startdoy == doy:
                        if(minValue < values[date] < maxValue):
                            isStation = True
                            self.AllStations.addStation(allData[station]["Meta"]["Lat"], allData[station]["Meta"]["Lon"], values[date], station)
                    continue
                '''
                if localtimeMin <= hour < localtimeMax and self.Startdoy == doy:
                    if(minValue < values[date] < maxValue):
                        isStation = True
                        allStations.addStation(allData[station]["Meta"]["Lat"], allData[station]["Meta"]["Lon"], values[date], station)

            if isStation:
                meanOfOneStation = allStations.getMeanOneStation(allData[station]["Meta"]["Lat"], allData[station]["Meta"]["Lon"])
                uniqueStations.addStation(allData[station]["Meta"]["Lat"], allData[station]["Meta"]["Lon"], meanOfOneStation, station)

        if uniqueStations.getCount() == 0:
            print "No Data found at " + str(self.Startdoy) + "_" + str(self.Time)
            #self.writeInLog("ERROR: No data in the timerange found")
            raise AssertionError("No data in the timerange found.")

        #stations with an error (missing Lat/Long value)
        if miss_station:
            print "WARNING: For this station(s) the metadata, in particular the latitude and longitude are missing: "
            print miss_station
            missing = ""
            for miss in miss_station:
                missing += str(miss) + "; "
            self.writeInLog("WARNING: For this station(s) the metadata, in particular the latitude and longitude are missing: " + missing)

        return uniqueStations

    def writeInLog(self, message):
        '''
        This function writes all massages in a log file.
        :param message: message to write
        :return:
        '''
        fobj = open(self.OutputfolderLog + self.LogFileName, "a")
        fobj.writelines(message + "\n")
        fobj.close()

    def getMinMaxBoundary(self, dict_lower, dict_upper, moy, station):
        '''
        This function searches for the maximal and minimal usable Data for hmF2 and NmF2. Reason: To find Outlyer.
        :param dict_lower: Dictionary from filter_option4.py
        :param dict_upper: Dictionary from filter_option4.py
        :param moy: Minute of the Year
        :param station: Station
        :return:
        '''

        interp_lower = dict_lower[station]
        interp_upper = dict_upper[station]
        data_bounds_lower = interp_lower(moy)
        data_bounds_upper = interp_upper(moy)

        if "NmF2" in self.Layer: return np.abs(data_bounds_lower) * 1e12, data_bounds_upper * 1e12
        else: return data_bounds_lower, data_bounds_upper

    def getDayOfMonthandMonth(self, doy):
        '''
        This function calculates the month and the day of the month from a day of year - value
        :param doy: day of year
        :return: day of month and month
        '''
        if doy == 1:
            mydate = str(datetime.datetime(self.Year, 1, 1) + datetime.timedelta(doy))
        else:
            mydate = str(datetime.datetime(self.Year, 1, 1) + datetime.timedelta(doy - 1))
        month = int(mydate[5: -12])
        dom = int(mydate[8: -9])
        return dom, month

    def getDayOfYear(self, dom, month, year):
        '''
        This function calculates the day of year.
        :param dom: day of month
        :param month: month
        :param year: year
        :return: day of year
        '''
        if year % 4 == 0: leapyear = 1
        else: leapyear = 0
        if month == 1: return dom
        elif month == 2: return 31 + dom
        elif month == 3: return 59 + leapyear + dom
        elif month == 4: return 90 + leapyear + dom
        elif month == 5: return 120 + leapyear + dom
        elif month == 6: return 151 + leapyear + dom
        elif month == 7: return 181 + leapyear + dom
        elif month == 8: return 212 + leapyear + dom
        elif month == 9: return 243 + leapyear + dom
        elif month == 10: return 273 + leapyear + dom
        elif month == 11: return 304 + leapyear + dom
        elif month == 12: return 334 + leapyear + dom
        else:
            self.writeInLog("ERROR: Something went horrible wrong! Could not find the month:" + str(month))
            raise AssertionError("Something went horrible wrong! Could not find the month:" + str(month))

    def convertMinInHour(self, min):
        '''
        This function converts minutes to hour
        :param min: minutes
        :return: hour
        '''
        return (1.0 / 60.0) * int(min)

    def getf107Values(self):
        '''
        This function gets the F10.7 values for the requested day of years in the config file
        :return:
        '''
        self.F107 = []
        try:
            for i in range(0, self.Enddoy + 1 - self.Startdoy):
                dom, month = self.getDayOfMonthandMonth(self.Startdoy + i)
                f107 = KPReader.KPFileReader(self.PathF107).getF10p7(self.Year, month, dom)
                self.F107.append(float(f107[0]))
        except:
            self.writeInLog("ERROR: KPFileReader had an Error! Solution: Check if the Path is correct, check if the correct files are available "
                                 "for the year, month and day you requested")
            raise AssertionError("ERROR: KPFileReader had an Error! Solution: Check if the Path is correct, check if the correct files are available "
                                 "for the year, month and day you requested")

    def residualsNPDMNPHM(self):
        '''
        This function reads the model value for NPDM and NPHM.
        :return:
        '''
        allDoys = []
        i = self.Startdoy
        while i < self.Enddoy + 1:
            allDoys.append(i)
            i += 1
        doy = np.array(np.asarray(allDoys), float)
        F10p7_sfu = np.array(self.F107, float)               #solar radio flux in flux units Vector

        meanAlt = []
        for i in range(0, self.StationshmF2.getCount()):
            lat_deg = np.ones(len(doy)) * self.StationshmF2.getLat(i)
            lon_deg = np.ones(len(doy)) * self.StationshmF2.getLon(i)
            UT_hrs = np.array([self.Hour], float)
            modelObj = NEDM.model(lat_deg, lon_deg, UT_hrs, doy, F10p7_sfu)
            nphm = modelObj.NPHM()
            meanAlt.append(float(nphm))
        self.StationshmF2.addResiduals(meanAlt)

        meanAlt = []
        for i in range(0, self.StationsNmF2.getCount()):
            lat_deg = np.ones(len(doy)) * self.StationsNmF2.getLat(i)
            lon_deg = np.ones(len(doy)) * self.StationsNmF2.getLon(i)
            UT_hrs = np.array([self.Hour], float)
            modelObj = NEDM.model(lat_deg, lon_deg, UT_hrs, doy, F10p7_sfu)
            npdm = modelObj.NPDM()
            meanAlt.append(float(npdm * (10**12)))
        self.StationsNmF2.addResiduals(meanAlt)

    def residualsNeQuick(self):
        '''
        This function calculates the difference of the read values and the values of the NeQuick model.
        :return:
        '''
        maxima = []
        for i in range(0, self.StationshmF2.getCount()):
            oneStation = {
            'height': np.asarray(self.AltList),
            'lat':  np.ones(len(self.AltList)) * self.StationshmF2.getLat(i),
            'lon': np.ones(len(self.AltList)) * self.StationshmF2.getLon(i),
            'month':self.Month,
            'f10p7':np.asarray(self.F107),
            'utcHour': self.Hour
            }
            wrapper = NeQuick.NeQuickWrapperVectorInput()
            wrapper.calcFromModel(oneStation)
            allValues = wrapper.modelData.values()[0]
            maximumValue = np.max(allValues)

            for i in range(len(allValues)):
                if allValues[i] == maximumValue:
                    maxima.append(self.AltList[i])
        self.StationshmF2.addResiduals(maxima)

        maxima = []
        for i in range(0, self.StationsNmF2.getCount()):
            oneStation = {
            'height': np.asarray(self.AltList),
            'lat':  np.ones(len(self.AltList)) * self.StationsNmF2.getLat(i),
            'lon': np.ones(len(self.AltList)) * self.StationsNmF2.getLon(i),
            'month':self.Month,
            'f10p7':np.asarray(self.F107),
            'utcHour': self.Hour
            }
            wrapper = NeQuick.NeQuickWrapperVectorInput()
            wrapper.calcFromModel(oneStation)
            allValues = wrapper.modelData.values()[0]
            maximumValue = np.max(allValues)
            maxima.append(maximumValue)
        self.StationsNmF2.addResiduals(maxima)

