# coding=utf-8

import numpy as np
import math

class Measurementstations(object):

    def __init__(self, doubleStations, layer, refstations=[]):
        '''
        Konstructor
        :param doubleStations: Boolean param. Allows double stations in the lat/lon list or not. True=allowed; False=not allowed
        :return:
        '''
        self.Lat = []
        self.Lon = []
        self.MeasurementValue = []
        self.DoubleStations = doubleStations
        self.StationID = []
        self.Residuals = []
        self.Layer = layer
        self.RefStations = refstations

    def addStation(self, lat, lon, measurement, stationid, residual=-9999):
        '''
        This function adds a station to the lat, lon and value list
        :param lat: Latitude of the station
        :param lon: Longitude of the station
        :param measurement: measurementvalue of the station
        :return:
        '''
        if self.DoubleStations:
            self.Lat.append(lat)
            self.Lon.append(lon)
            self.MeasurementValue.append(measurement)
            self.StationID.append(stationid)
            if residual != -9999: self.Residuals.append(residual)
        else:
            isDouble = False
            for i in range(len(self.Lat)):
                if self.calcDistSpherical(lat, lon, self.Lat[i], self.Lon[i]) <= 30:
                    isDouble = True
                    break
            if not isDouble:
                self.Lat.append(lat)
                self.Lon.append(lon)
                self.MeasurementValue.append(measurement)
                self.StationID.append(stationid)
                if residual != -9999: self.Residuals.append(residual)

    def getStation(self, position):
        '''
        Returns a station on a specific position in the list
        :param position: position in the list
        :return: lat, lon and measurement value of the requested station
        '''
        return self.Lat[position], self.Lon[position], self.MeasurementValue[position]

    def getLat(self, position):
        '''
        Return the latitude of one station in the list
        :param position: position of the station in the list
        :return: latitude of the station
        '''
        return self.Lat[position]

    def getLon(self, position):
        '''
        Return the longitude of one station in the list
        :param position: position of the station in the list
        :return: longitude of the station
        '''
        return self.Lon[position]

    def getAllStationID(self):
        '''
        This function returns all station IDs.
        :return: List of Station IDs
        '''
        return self.StationID

    def getStationID(self, position):
        '''
        This function returns the ID of on station in the list.
        :param position: ID of the station
        :return:
        '''
        return self.StationID[position]

    def getPosOfStation(self, id):
        '''
        This function returns the position of the station id in the internal list of this object.
        :param id: station id
        :return: position
        '''
        for i in range(len(self.StationID)):
            if self.StationID[i] == id: return i

    def getMeasurement(self, position):
        '''
        Return the measurement value of one station in the list
        :param position: position of the station in the list
        :return: measurement value of the station
        '''
        return self.MeasurementValue[position]

    def getMeasurementFromID(self, id):
        '''
        This function returns the measurement value from a station. If the station is not found, -1 will be returned.
        :param id: station id
        :return: Measurement value or -1, if the station id is part of the List
        '''

        for i in range(len(self.StationID)):
            if id == self.StationID[i]:
                return self.getMeasurement(i)

        return -1

    def getAllStation(self):
        '''
        Returns all stations in the list
        :return: lat, lon and measurement value of all stations
        '''
        return self.Lat, self.Lon, self.MeasurementValue

    def getAllLat(self):
        '''
        Return latitudes for all stations
        :return: list of latitudes
        '''
        return self.Lat

    def getAllLon(self):
        '''
        Return longitudes for all stations
        :return: list of longitudes
        '''
        return self.Lon

    def getAllMeasurement(self):
        '''
        Return measurement values for all stations
        :return: list of measurement values
        '''
        return self.MeasurementValue

    def getMean(self):
        '''
        This function calculates the mean of all measurement values
        :return: mean of all measurement values
        '''
        return np.mean(self.MeasurementValue)

    def getCount(self):
        '''
        This function counts the stations
        :return: count of stations
        '''
        return len(self.MeasurementValue)

    def deleteOneStation(self, lat, lon):
        '''
        This function deletes a station from the list
        :param lat: latitude of the station
        :param lon: longitude of the station
        :return:
        '''
        position = -1
        for i in range(len(self.Lon)):
            if self.Lon[i] == lon and self.Lat[i] == lat:
                position = i
                break
        if position != -1:
            del self.Lat[position]
            del self.Lon[position]
            del self.MeasurementValue[position]
        else:
            print "Station could not be deleted, because it is not in the list! Coordinates ", lat, lon

    def clacfof2TONmF2(self):
        for i in range(len(self.MeasurementValue)):
            self.MeasurementValue[i] = (1.24 * 10**10) * (self.MeasurementValue[i]**2)

    def getMeanOneStation(self, lat, lon):
        '''
        This function calculates the mean of every measurement value for one station.
        :param lat: latitude of the station
        :param lon: longitude of the station
        :return: 0, if the station was not found
        '''
        mean = 0
        counter = 0
        for i in range(len(self.Lat)):
            if lat == self.Lat[i] and lon == self.Lon[i]:
                mean += self.MeasurementValue[i]
                counter += 1
        if counter > 0:
            return mean / counter
        else:
            print "WARNING: Could not find the coordinates of the station in the list -> Could not calculate the mean! Returned 0"
            return 0

    def diffModel(self, modelValues):
        '''
        This function calculates the difference between the measurement values and the model values
        :param modelValues: list of all modelvalues
        :return: list of differences between measurement value and model values
        '''
        diff = []
        if len(modelValues) != len(self.MeasurementValue):
            raise AssertionError("Count of the model values is not equal to the count of the measurement values! Program stopped!")

        for i in range(len(modelValues)):
            diff.append(self.MeasurementValue[i] - modelValues[i])

        return diff

    def diffModelRelative(self, modelValues):
        '''
        This function calculates the relative difference between the measurement values an the model values ((realvalue-referencevalue)/referencevalue)
        :param modelValues: list of modelvalues
        :return: list of relative differences
        '''
        diff = []
        if len(modelValues) != len(self.MeasurementValue):
            raise AssertionError("Count of the model values is not equal to the count of the measurement values! Program stopped!")

        for i in range(len(modelValues)):
            diff.append(np.abs((self.MeasurementValue[i] - modelValues[i]) / modelValues[i]))

        return diff

    def removeRefStations(self):
        '''
        This function returns a list of all Stations without the reference stations.
        :param refstationid: List of reference station IDs
        :return: a list of all Stations without the reference stations
        '''

        station = Measurementstations(False, self.Layer)
        for i in range(len(self.StationID)):
            if not self.StationID[i] in self.RefStations:
                station.addStation(self.Lat[i], self.Lon[i], self.MeasurementValue[i], self.StationID[i], residual=self.Residuals[i])

        return station

    def addResiduals(self, residuals):
        '''
        This functions adds the residuals to the MeasurementStation - object.
        :param residuals: Residuals datatyp:list
        :return:
        '''
        self.Residuals = residuals

    def getResiduals(self):
        return self.Residuals

    def getLayer(self):
        return self.Layer

    def getRefStations(self):
        station = Measurementstations(False, self.Layer)
        for i in range(len(self.StationID)):
            if self.StationID[i] in self.RefStations:
                station.addStation(self.Lat[i], self.Lon[i], self.MeasurementValue[i], self.StationID[i])

        return station

    def calcDistSpherical(self, lat1, lon1, lat2, lon2):
        '''
        This function calculates the spherical distance between two points. Haversine formula used.
        :param lat1: latitude of origin
        :param lon1: longitude of origin
        :param lat2: latitude of destination
        :param lon2: longitude of destination
        :return: spherical distance of the two points
        '''
        if lat1 == lat2 and lon2 == lon1: return 0.0
        dlat = np.radians(lat2-lat1) / 2
        dlon = np.radians(lon2-lon1) / 2
        a = np.sin(dlat) * np.sin(dlat) + np.cos(np.radians(lat1)) * np.cos(np.radians(lat2)) * np.sin(dlon) * np.sin(dlon)
        c = 2 * math.atan2(np.sqrt(a), np.sqrt(1-a))
        return 6378 * c