#!/usr/bin/env python

"""
Module name:		
	NEDMv1.py
Description:	
	Neustrelitz Electron Density Model (NEDM) is coded in Python. The NEDM is developed 
	combining different 2D ionosphere key parameter models such as Neustrelitz TEC Model (NTCM), 
	Neustrelitz Peak Density Model (NEDM), Neustrelitz Peak Height Model (NTHM), 
	Plasmaspheric basic density model and plasmaspheric scale height model etc.
	In the present version the 3D approach is realized using a single layer Chapman profile function 
	with a superposed plasmasphere function. 
Version:	v1
Author:		Mainul Hoque
Created:	27.11.2014
Copyright:	(c) Mainul Hoque 2014
Mail:		Mainul.Hoque@dlr.de
Licence:	DLR
"""
import numpy as np
from bin.reconstruct.ModelNPDM_NPHM import ionofunclib as myfunc
import bin.reconstruct.ModelNPDM_NPHM.ionoconslib as mycons
import pylab as pl
	
	
class ionpar():
	"""
	usage: used for subclass model(ionpar) initialization
	Syntax:
		ionparmodelObj = ionpar()
		LT = ionparmodelObj.LT()
		latm_rad = ionparmodelObj.latm_rad()
		cos_solzenith = ionparmodelObj.cos_solzenith()
		szfact = ionparmodelObj.szfact()
		cosDVmsz = ionparmodelObj.cosDVmsz()
		cosDVsz = ionparmodelObj.cosDVsz()
		sinDVsz = ionparmodelObj.sinDVsz()
		cosSDVsz = ionparmodelObj.cosSDVsz()
		sinSDVsz = ionparmodelObj.sinSDVsz()
		cosTDVsz = ionparmodelObj.cosTDVsz()
		sinTDVsz = ionparmodelObj.sinTDVsz()		
	Description:
		optional inputs:
		lat_deg: geographic latitude Vector in degrees
	    lon_deg: geographic longitude Vector in degrees
	    UT_hrs: universal time (UT) Vector in hours
	    doy: day of year Vector (or scaler) 
	    F10p7_sfu: solar radio flux in flux units Vector (or scaler)
		default inputs:	lat_deg = 0, lon_deg = 0, UT_hrs = 0, doy = 1, F10p7_sfu = 100
		LT = Local Time Vector in hours
		latm_rad = geomagnetic latitude Vector in radians
		cos_solzenith = cosine of solar zenith angle Vector
		szfact = solar zenith factor Vector
		cosDVmsz = cosine of modified diurnal variation multiply with szfact Vector
		cosDVsz = cosine of diurnal variation multiply with szfact Vector
		sinDVsz = sine of diurnal variation multiply with szfact Vector
		cosSDVsz = cosine of semi-diurnal variation multiply with szfact Vector
		sinSDVsz = sine of semi-diurnal variation multiply with szfact Vector
		cosTDVsz = cosine of ter-diurnal variation multiply with szfact Vector
		sinTDVsz = sine of ter-diurnal variation multiply with szfact Vector		
	"""
	def __init__(self, lat_deg = 0, lon_deg = 0, UT_hrs = 0, doy = 1, F10p7_sfu = 100):
		self.lat_deg            = lat_deg
		self.lat_rad 	= np.deg2rad(self.lat_deg) 
		self.lon_deg		= lon_deg
		self.UT_hrs	        = UT_hrs
		self.doy		= doy
		self.F10p7              = F10p7_sfu
		self.LT = myfunc.UT2LT(self.UT_hrs, self.lon_deg)		
		self.latm_rad = myfunc.geoLat2magLat(self.lat_rad, self.lon_deg)
		cos_solzenith, self.szfact =  myfunc.sunZenith(self.doy, self.lat_rad)
		
		self.cosDVmsz = np.cos(2. * np.pi * (self.LT - 14.)/24.) * cos_solzenith
		self.cosDVsz = np.cos(2. * np.pi * self.LT/24.) * cos_solzenith
		self.sinDVsz = np.sin(2. * np.pi * self.LT/24.) * cos_solzenith
		self.cosSDVsz = np.cos(2. * np.pi * self.LT/12.) * cos_solzenith
		self.sinSDVsz = np.sin(2. * np.pi * self.LT/12.) * cos_solzenith		
		self.cosTDVsz = np.cos(2. * np.pi * self.LT/8.) * cos_solzenith
		self.sinTDVsz = np.sin(2. * np.pi * self.LT/8.) * cos_solzenith		
		del self.lat_deg, self.lon_deg, self.UT_hrs    
		
	def __repr__(self):
	    return "<initial value: lat_deg=%s, LT_hrs=%s, doy=%s, F10.7=%s, latm_rad=%s, solzenith_factor=%s>" % (np.rad2deg(self.lat_rad), self.LT, self.doy, self.F10p7, self.latm_rad, self.szfact )   

class model(ionpar):
	"""
	Syntex:
		modelObj = model( lat_deg, lon_deg, UT_hrs, doy, F10p7_sfu )
		TEC = modelObj.NTCM()
		NmF2 = modelObj.NPDM()
		hmF2 = modelObj.NPHM()
		Np0 = modelObj.Np0()
		Hplasma = modelObj.Hplasma()
	Description:	
	     lat_deg: geographic latitude Vector in degrees
	     lon_deg: geographic longitude Vector in degrees
	     UT_hrs: universal time (UT) Vector in hours
	     doy: day of year Vector (or scaler) 
	     F10p7_sfu: solar radio flux in flux units Vector (or scaler)
		 TEC = vertical TEC Vector up to GNSS altitude in TECU
		 NmF2 = F2 layer peak density Vector in 1.e+12 el/m^3
		 hmF2 = F2 layer peak density height Vector in km
		 Np0 = plasmaspheric basic density Vector in 1.e+12 el/m^3
         Hplasma = plasmaspheric scale height Vector in km		
	"""
	def __init__(self, lat_deg, lon_deg, UT_hrs, doy, F10p7_sfu):
		ionpar.__init__(self,lat_deg, lon_deg, UT_hrs, doy, F10p7_sfu)
		
	    
	def NTCM(self):
		
		"""
		Syntax:
			modelObj = model( lat_deg, lon_deg, UT_hrs, doy, F10p7_sfu )
			TEC = modelObj.NTCM()
		Description:
			TEC: Total Electron Content in TECU (Vertical direction up to GPS height)
			for more see description of model(ionpar)
					
			NeustrelitZ Total Electron Content Model (NTCM) approach
			Local Time variation:
			   Diurnal Variation: DVm = 2*pi*(LT-14)/24
			   Semi-Diurnal Variation: SDV = 2*pi*LT/12
			   Tar-Diurnal Variation: TDV = 2*pi*LT/8
			   F1 = cos(chi)***+((c1cos(DVm)+c2sin(DV)+c3cos(SDV)+c4sin(SDV)+c5cos(TDV)+c6sin(TDV))cos(chi)**
			   cos(chi)*** 	= cos(phi - delta) + 0.4 # square root is omitted, it was a printing mistake in Jakowski et al .[2011] 
			   cos(chi)** = cos(phi - delta) - 2phi/pi*sin(delta); phi = geographic latitude radian, delta = sun declination radian
			Annual Semiannual variation
			   Annual Variation: AV = 2*pi(doy-doy_DAV)/365.25; doy_DAV = 18
			   Semi-Annual Variation: SAV = 4*pi(doy-doy_DSAV)/365.25; doy_DSAV = 6
			   F2 =1+c7cos(AV)+c8cos(SAV)
			Dependence from geomagnetic field
			   F3 =1+c9cos(phi_m); phi_m = geomagnetic latitude radian
			Dependence from Crest region
			   Equatorial Crest: EC1 = (phi_m-phi_c1)^2/2sigma_1^2; phi_c1 = 16N
			   Equatorial Crest: EC2 = (phi_m-phi_c2)^2/2sigma_2^2; phi_c2 = -10N
			   F4 = 1+c10exp(EC1)+c11exp(EC2)
			Dependence from solar activity
			   F5 =c12+c13*F10.7; F10.7 = solar radio flux in flux units
			   TECvt = F1*F2*F3*F4*F5 in TECU 
			REFERENCE: Jakowski, N. and Hoque, M. M. and Mayer, C. (2011) 
			           A new global TEC model for estimating transionospheric radio wave propagation errors. 
			           Journal of Geodesy, 85 (12), pp. 965-974. Springer. DOI: 10.1007/s00190-011-0455-1. ISSN 0949-7714. 
		"""
		
		K = (0.8966, 0.1698, -0.0217, 0.0593, 0.0074, 0.1391, -0.1759, -0.3455, 1.1168, 1.1573, -4.3356, 0.1778)
		
		return ((self.szfact + K[0] * self.cosDVmsz + K[1] * self.cosSDVsz + K[2] * self.sinSDVsz + K[3] * self.cosTDVsz + K[4] * self.sinTDVsz) * 
					(1 + K[5] * np.cos(2 * np.pi * (self.doy - 18.)/365.25) + K[6] * np.cos(4 * np.pi * (self.doy - 6.)/365.25)) * 
					(1 + K[7] * np.cos(self.latm_rad)) * (1 + K[8] * np.exp(-(np.rad2deg(self.latm_rad) - 16.) **2 / 288.) + K[9] * np.exp(-(np.rad2deg(self.latm_rad) + 10.) **2 / 338.)) * 
					(K[10] + K[11] * self.F10p7))
		
	    
	def NPDM(self):
		"""
		Syntax:
			modelObj = model( lat_deg, lon_deg, UT_hrs, doy, F10p7_sfu )
			NmF2 = modelObj.NPDM()
		Description:
			NmF2: Maximum ionization of F2 layer in 1.e+12 el/m^3
			for more see description of model(ionpar)
		
		Neustrelitz Peak Density Model (NPDM) approach
		Local Time variation:
			Diurnal Variation: DVm = 2*pi*(LT-14)/24
			Semi-Diurnal Variation: SDV = 2*pi*LT/12
			Tar-Diurnal Variation: TDV = 2*pi*LT/8
			F1 = cos(chi***)+((c1cos(DVm)+c2cos(SDV)+c3sin(SDV)+c4cos(TDV)+c5sin(TDV))cos(chi**)+c13BO
			cos(chi)*** 	= cos(phi - delta) + 0.4 
			cos(chi)** = cos(phi - delta) - 2phi/pi*sin(delta); phi = geographic latitude radian, delta = sun declination radian
	        Midlatitude Summer daytime Bite Out effect: BO
				sigma_BO = 3;
				doyBO = 181;
				LTBO = 13. + 1.5*cos(2*pi*(doy - doyBO)/365.25).*SIGN;  
				EBO = exp(-(LT-LTBO).^2/2/sigma_BO^2);
				sigma_BO2 = 14;
				EBO2 = exp(-(abs(lat_deg) - 45).^2/2/sigma_BO2^2).*SIGN;
				BO = EBO2.*cos(2*pi*(doy-doyBO)/365.25).*EBO;			
		Annual Semiannual variation
			Annual Variation: AV = 2*pi(doy - doy_DAV)/365.25; doy_DAV = 18
			Semi-Annual Variation: SAV = 4*pi(doy - doy_DSAV)/365.25; doy_DSAV = 6
			F2 =1+c7cos(AV)+c8cos(SAV)
		Dependence from geomagnetic field
			F3 =1+c9cos(phi_m); phi_m = geomagnetic latitude
		Dependence from Crest region
			phi_c1 = 16; %degrees                                                 
			phi_c2 = -15; %degrees
			sigma_LT = 12;
			EC_LT = -(LT - 14).^2/2/sigma_LT^2;
			sigma_c1 = 20 - 10*exp(EC_LT);
			sigma_c2 =  sigma_c1;
			Equatorial Crest: EC1 = (phi_m-phi_c1)^2/2sigma_1^2
			Equatorial Crest: EC2 = (phi_m-phi_c2)^2/2sigma_2^2
			F4 = 1+c10exp(EC1)+c11exp(EC2)
		Dependence from solar activity
			F5 =c12+c13*F10.7
		NmF2 = F1*F2*F3*F4*F5 in 1.e+12 el/m^3 
		Reference:
			Hoque, M. M. and Jakowski, N.: A new global empirical NmF2
				model for operational use in radio systems, Radio Sci., 46,
				RS6015, doi:10.1029/2011RS004807, 2011b.			
		"""
		K = (1.00889, 0.12569, -0.00882, 0.10268, 0.04894, 0.13443, -0.16045, 0.19216, 0.62074, 0.58005, 0.91756, -1.18833, -0.83712)

		SIGN = self.lat_rad/np.fabs(self.lat_rad);
		SIGN = np.select([ np.isnan(SIGN), ~np.isnan(SIGN)], [0, SIGN])
		
		
		BO = ((np.exp(-(np.fabs(np.rad2deg(self.lat_rad)) - 45.)**2/392.)*SIGN) * 
			np.cos(2*np.pi*(self.doy - 181.)/365.25) * np.exp(-(self.LT - (13. + 1.5*np.cos(2*np.pi*(self.doy - 181.)/365.25)*SIGN))**2/18.))
    
		sigma_c1 = 20. - 10.*np.exp(-(self.LT - 14.)**2/288.)
		
		return ((self.szfact + K[0]*self.cosDVmsz + K[1]*self.cosSDVsz + K[2]*self.sinSDVsz + K[3]*self.cosTDVsz + K[4]*self.sinTDVsz + K[12]*BO)*
					(1 + K[5] * np.cos(2 * np.pi * (self.doy - 18.)/365.25) + K[6] * np.cos(4 * np.pi * (self.doy - 6.)/365.25)) * (1 + K[7]*np.cos(self.latm_rad)) * 
					(1 + K[8] * np.exp(-(np.rad2deg(self.latm_rad) - 16.) **2 / 2. / sigma_c1**2) + K[9] * np.exp(-(np.rad2deg(self.latm_rad) + 15.) **2 / 2. / sigma_c1**2)) * 
					(K[10] + K[11]*np.exp(-self.F10p7/144.)))
							
		
	def NPHM(self):
				
		"""
		Syntax:
			modelObj = model( lat_deg, lon_deg, UT_hrs, doy, F10p7_sfu )
			hmF2 = modelObj.NPHM()
		Description:
			hmF2: Maximum ionization height of F2 layer in km
			for more see description of model(ionpar)
		
		Neustrelitz Peak Height Model (NPHM) approach
		Local Time variation:
			Diurnal Variation: DV = 2*pi*LT/24
			Semi-Diurnal Variation: SDV = 2*pi*LT/12
			Tar-Diurnal Variation: TDV = 2*pi*LT/8
			F1 = 1 + c1*cos(chi***)+(c2*cos(DV)+c3*sin(DV)+c4*cos(SDV)+c5*sin(SDV)+c6*cos(TDV)+c7*sin(TDV))cos(chi**)
			cos(chi)*** 	= cos(phi - delta) + 0.4  
			cos(chi)** = cos(phi - delta) - 2phi/pi*sin(delta); phi = geographic latitude radian, delta = sun declination radian
	    Annual Semiannual variation
			Annual Variation: AV = 2*pi(doy - doy_DAV)/365.25; doy_DAV = 181
			Semi-Annual Variation: SAV = 4*pi(doy - doy_DSAV)/365.25; doy_DSAV = 49	
			F2 = 1+c8*cos_AV+c9*cos_SAV	
		Dependence from geomagnetic field
			sigma_c2 = 40; %degrees
			exp_EC_phi = exp(-latm_deg.^2/2/sigma_c2^2);      
			sigma_c1 = 20; %degrees
			EC_phi = -latm_deg.^2/2/sigma_c1^2;
			sigma_LT = 4;
			EC_LT = -(LT - 14).^2/2/sigma_LT^2;
			exp_phi_LT = exp(EC_phi).*exp(EC_LT);
			F3 = 1+c10*exp_EC_phi+c11*exp_phi_LT
       Dependence from solar activity
			delta_F10p7 = 10.8;
			exp_F10p7 = exp(-F10p7/delta_F10p7^2);
            F4 = c12+c13*exp_F10p7  
	   hmF2 = F1 * F2 *F3 *F4 	
		Reference:
			Hoque, M. M. and Jakowski, N.: A new global model for the ionospheric F2 peak height for radio
				wave propagation, Ann. Geophys., 30, 797809, 2012, www.ann-geophys.net/30/797/2012/
				doi 10.5194/angeo-30-797-2012
		"""
		
		K = (0.09246, 0.19113, 0.02297, 0.05666, -0.01687, -0.01590, 0.01194, -0.01781, -0.00618, -0.14070, 0.46728, 348.66432, -184.15337)
		
		
		return ((1 + K[0]*self.szfact + K[1]*self.cosDVsz + K[2]*self.sinDVsz + K[3]*self.cosSDVsz + K[4]*self.sinSDVsz + K[5]*self.cosTDVsz + K[6]*self.sinTDVsz) *
					(1 + K[7] * np.cos(2 * np.pi * (self.doy - 181.)/365.25) + K[8] * np.cos(4 * np.pi * (self.doy - 49.)/365.25)) * 
					(1 + K[9] * np.exp(-np.rad2deg(self.latm_rad)**2/3200.) + K[10] * np.exp(-np.rad2deg(self.latm_rad)**2/800.)*np.exp(-(self.LT - 14.)**2/32.)) * 
					(K[11] + K[12] * np.exp(-self.F10p7/116.64))) 

	def Np0(self):
		
		"""
		Syntax:
			modelObj = model( lat_deg, lon_deg, UT_hrs, doy, F10p7_sfu )
			Np0 = modelObj.Np0()
		Description:
			Np0 basic ionization of plasmaspheric in 1.e+12 el/m^3
			for more see description of model(ionpar)
		
		Plasmaspheric basic density (Np0) model approach
		Local Time variation:
		   Diurnal Variation: DV = 2*pi*LT/24
		   F1 = cos(chi***)^2+((c1cos(DV)+c2sin(DV))cos(chi**)
		Dependence from geomagnetic field
		   F3 =1+c3cos(phi_m)
		Dependence from Crest region
		   Equatorial Crest: EC1 = (phi_m-phi_c1)^2/2sigma_1^2
		   phi_c1=0;%degrees                                                 
		   sigma_c1=21;%degrees
   		   F4 = 1+c4exp(EC1)
		Dependence from solar activity
		   F5 =c5+c6*F10.7
		   Np0 = F1*F3*F4*F5 in 1.e+12 el/m^3 
		"""
		
		K = (-0.81995, 0.58153, 22.09993, 0.33240, 0.00096, -0.00000)
	
		return ((self.szfact + K[0] * self.cosDVsz + K[1] * self.sinDVsz) * (1 + K[2] * np.cos(self.latm_rad)) * 
					(1 + K[3] * np.exp(-np.rad2deg(self.latm_rad)**2/882.)) * 
					(K[4] + K[5] * self.F10p7)) 
        
	def Hplasma(self):
		
		"""
		Syntax:
			modelObj = model( lat_deg, lon_deg, UT_hrs, doy, F10p7_sfu )
			Hplasma = modelObj.Hplasma()
		Description:
			NZHplasma = plasmaspheric scale height in km
			for more see description of model(ionpar)
		
		PLasmaspheric scale height (Hplasma) model approach
		Local Time variation:
		   Diurnal Variation: DV = 2*pi*LT/24
		   F1 = cos(chi***)^2+((c1cos(DV)+c2sin(DV))cos(chi**)
		Dependence from geomagnetic field
		   F3 =1+c3cos(phi_m)
		Dependence from Crest region
		   Equatorial Crest: EC1 = (phi_m-phi_c1)^2/2sigma_1^2
		   phi_c1=0;%degrees                                                 
		   sigma_c1=28;%degrees
   		   F4 = 1+c4exp(EC1)
		Dependence from solar activity
		   F5 =c5+c6*F10.7
		   NZHplasma = F1*F3*F4*F5 in km
		"""
				
		K = (0.27014, -0.31956, -0.29536, 1.72974, 1262.81781, -0.59904)
		
		return ((self.szfact + K[0] * self.cosDVsz + K[1] * self.sinDVsz) *
				(1 + K[2] * np.cos(self.latm_rad)) *
				(1 + K[3] * np.exp(-np.rad2deg(self.latm_rad)**2/1568.)) * 
				(K[4] + K[5] * self.F10p7))
		
class stec(model):
	"""
		Syntax:
			stecObj = stec( x1, y1, z1, x2, y2, z2, UT_hrs, doy, F10p7_sfu, parset['IonLimitm'], parset['IonStepm'], parset['PlasmaStepm'])
			STEC = stecObj.stec()
			Ne, altkm = stecObj.getNe()
		Description:
			x1 = ECEF x coordinate of point P1 (e.g., Rx at ground)
			y1 = ECEF y coordinate of point P1
			z1 = ECEF z coordinate of point P1
			x2 = ECEF x coordinate of point P2 (e.g., GNSS Tx)
			y2 = ECEF y coordinate of point P2
			z2 = ECEF z coordinate of point P2
			UT_hrs = universal time in hours
			doy = day of year
			F10p7_sfu = solar radio flux F10.7 in flux units
			optional parameters:
			IonLimitm = altitude limit of ionosphere in meter (above Earth's surfface)
			IonStepm = step size of each layer in meter 
			PlasmaStepm = step size of the plasmasphere which extends from IonLimitm until Tx height
			default values:
			parset 	= {	"IonLimitm" : 1000000.,	"IonStepm" : 10000,	"PlasmaStepm" : 50000}
			STEC = slant TEC connecting point P1 & P2 in TECU
			Ne = electron density Vector in 10**12 el/m**3
			altkm = corresponding altitude Vector in km  
	"""	
	
	def __init__(self, x1m, y1m, z1m, x2m, y2m, z2m, UThrs, doy, F10p7sfu, IonLimitm = 1000000., IonStepm = 10000.,	PlasmaStepm = 50000.):
		
		parset 	= {	"IonLimitm" : IonLimitm,	"IonStepm" : IonStepm,	"PlasmaStepm" : PlasmaStepm}
		xm, ym, zm, self.dlm = myfunc.rayinterpoints(x1m, y1m, z1m, x2m, y2m, z2m, parset)	
		latrad, lonrad, self.altm = myfunc.ecef2geodetic(xm, ym, zm)
				
		model.__init__(self,np.rad2deg(latrad), np.rad2deg(lonrad), UThrs, doy, F10p7sfu)
		
		
	def stec(self):		
		"""
		Syntax:
			stecObj = stec( x1, y1, z1, x2, y2, z2, UT_hrs, doy, F10p7_sfu, parset['IonLimitm'], parset['IonStepm'], parset['PlasmaStepm'])
			STEC = stecObj.stec()	
		Description:
			STEC = slant TEC connecting point P1 & P2 in TECU			
		"""
		Ne, altkm = self.getNe()
		#calculate slant TEC in TECU		
		return np.sum(Ne*self.dlm)/1.e+4                                
    
		
	def getNe(self):
		"""
		Syntax:
			stecObj = stec( x1, y1, z1, x2, y2, z2, UT_hrs, doy, F10p7_sfu, parset['IonLimitm'], parset['IonStepm'], parset['PlasmaStepm'])
			STEC = stecObj.stec()	
		Description:
			Ne, altkm = stecObj.getNe()	
			Ne = electron density Vector in 10**12 el/m**3
			altkm = corresponding altitude Vector in km		
		"""
		#get 2D model parameters
		TECvt = self.NTCM() 											#total vertical TECU until GPS height
		NmF2 = self.NPDM()                                              #peak density in 10**12 el/m**3
		hmF2 = self.NPHM()                                              #peak dnesity height in km
		Np0 = self.Np0()                                                #plasmaspheric basic density in 10**12 el/m**3 
		Hplasma = self.Hplasma()                                        #plasmaspheric scale height in km
		TECvb = TECvt * 0.3173                                          #bottomside TEC in TECU applying Chapman approximation: 
		                                                                #erf(-exp(-(Sh-hm))/sqrt(2)) - erf(-exp(-(Rh-hm))/sqrt(2)); 
		                                                                #Sh = hm, Rh = 0, then the factor becomes 0.317
		h = self.altm/1000
		latm_rad = self.latm_rad
		NmE = 0.                                                        #set to zero for the current version
		Lpp = 5
		N_MS = 1.e-6                                                    #in 10^1^2 el/m^3;
		HE = 15                                                         #in km
		TEC_Elayer = 4.13*NmE*HE/10.                                    #TECU   
		HF2_bottom = TECvb/NmF2/1.3105*10.                              # in km
    
		#plasmaspheric content PEC
		a = -np.exp(-20200./Hplasma) + 300./(Hplasma+300.) * np.exp(-(20200. - 300.)/300.)*np.exp(-20200./Hplasma)
		b = -np.exp(-300./Hplasma) + 300./(Hplasma+300.) * np.exp(-300./Hplasma)
		PEC = Np0*Hplasma*(a-b)/10.                                     #in TECU
    
    
		TEC_top = TECvt - TECvb - TEC_Elayer - PEC                      #Chapman layer topside TEC  
		HF2_top = TEC_top/2.8195/NmF2*10                                # in km 
    
		hmE = 108.5                                                     #in km
		zE = (h - hmE)/HE
		NeE = NmE*np.exp(0.5*(1-zE - np.exp(-zE)))                      #in e+12 el/m^3
		
		NeF2 = np.zeros(np.shape(NmF2))
		indices = np.nonzero(h <= hmF2)
		z = (h[indices] - hmF2[indices])/HF2_bottom[indices]
		NeF2[indices] = NmF2[indices]*np.exp(0.5*(1-z-np.exp(-z)))     #in e+12 el/m^3
    
		indices = np.nonzero(h > hmF2)
		z = (h[indices] - hmF2[indices])/HF2_top[indices]
		NeF2[indices] = NmF2[indices]*np.exp(0.5*(1-z-np.exp(-z)))     #in e+12 el/m^3
    
    
		indices = np.nonzero(h > 300)
		z = (h[indices] - 300)/300
		F1 = 1 - np.exp(-z)
		F2 = Np0[indices]*np.exp(-h[indices]/Hplasma[indices])
		Rpp = Lpp*mycons.REm/1000*np.cos(latm_rad[indices])**2
    
		h_tmp = h[indices]
		indices_tmp = np.nonzero(h_tmp < 1000)
		h_tmp[indices_tmp] = 1000                                       
    
		F3 = np.arctan((Rpp - mycons.REm/1000 - h_tmp)/200)/np.pi+0.5   
		Np = np.zeros(np.shape(h))
		Np[indices] = F1*(F2*F3 + N_MS)
    	
		return (NeF2 + NeE + Np), h 	





if __name__ == "__main__":
	
	"""
	Testing each class / method
	"""
	ionparmodelObj = ionpar()
	print(ionparmodelObj)
	lat_deg = np.array([10, 70],float) 
	lon_deg = np.array([45, 50],float) 
	UT_hrs = np.array([10, 15],float) 
	doy = np.array([10, 15],float) 
	F10p7_sfu = np.array([80, 100],float) 
	
	modelObj = model( lat_deg, lon_deg, UT_hrs, doy, F10p7_sfu )
	print "TEC [TECU]:", modelObj.NTCM()
	print "NmF2 [1.e+12 el/m3]:", modelObj.NPDM()
	print "hmF2 [km]:", modelObj.NPHM()
	print "Np0 [1.e+12 el/m3]:", modelObj.Np0()
	print "Hplasma [km]:", modelObj.Hplasma()
	
	
	x1, y1, z1 = myfunc.geodetic2ecef(np.deg2rad(lat_deg), np.deg2rad(lon_deg), np.array([1., 1.],float))
	print "x [km]: %f  y [km]: %f z [km]: %f", x1/1000, y1/1000, z1/1000
	
	latrad, lonrad, altm = myfunc.ecef2geodetic(x1, y1, z1)
	print "lat [deg]: %f  lon [deg]: %f alt [km]: %f", np.rad2deg(latrad), np.rad2deg(lonrad), altm/1000
	
	x2, y2, z2 = myfunc.geodetic2ecef(np.deg2rad(lat_deg), np.deg2rad(lon_deg), np.array([20200000, 20200000],float))
	print "x [km]: %f  y [km]: %f z [km]: %f", x2/1000, y2/1000, z2/1000
	
	parset 	= {	"IonLimitm" : 1000000.,	"IonStepm" : 20000.,	"PlasmaStepm" : 50000.}
	stecObj = stec( x1[0], y1[0], z1[0], x2[1], y2[1], z2[1],UT_hrs[0], doy[0], F10p7_sfu[0], parset['IonLimitm'], parset['IonStepm'],	parset['PlasmaStepm'])
	print "TEC [TECU]:", stecObj.stec()
	Ne, altkm = stecObj.getNe()
	print 'max alt km', np.amax(altkm)
	pl.plot(Ne, altkm)
	pl.xlabel('Electron Density [1.e+12 el/$m^3$]')
	pl.ylabel("Altitude [km]")
	pl.ylim(0, 1000)	
	pl.show()
	
	
	
