#!/usr/bin/env python
#-------------------------------------------------------------------------------
# Name:         NeQuickIonosondeWrapper.py
#
# Purpose:      This software wraps the compiled NeQuick Fortran model including assimilation of ionosonde key parameters hmF2 and foF2.
#
#               This software was developed to use the 3D electron density model NeQuick from python software.
#
# Needs:        python, f2py, gfortran, NeQuickIonosonde.so, 
#				assimilated foF2 and hmF2 maps (/data1/mink_da/3DReconstruction/SCM)
#
# Version:      $Id: PIMWrapper.py 7470 2013-09-25 12:07:01Z krie_ma $
#               $HeadURL: https://svn.dlr.de/GNSSsoft/trunk/NZ_Projects/IMPC/Tools/Python/NeQuickPythonInterface/NeQuick_PythonEnvironment/NeQuickIonosondeWrapper.py $
#
# Author:       Tatjana Gerzen
#				David Minkwitz
#				Martin Kriegel
#
# Created:      04.11.2013
# Copyright:    (c) Tatjana Gerzen & David Minkwitz & Martin Kriegel & 2013
# Mail:         Tatjana.Gerzen@dlr.de David.Minkwitz@dlr.de Martin.Kriegel@dlr.de 
# Licence:      DLR
#-------------------------------------------------------------------------------

import NeQuickIonosonde, os

class NeQuickIonosondeWrapper():
    """
    This class wraps the shared library NeQuick_2_0_2.so, which was compiled with gfortran and f2py
    during the build of the environment. The input parameters are wrapped in the dictionary data. The
    requested data from NeQuick_2_0_2 is accessible via the public member 'modelData', which is also
    a dictionary. A description for the 'modelData' is accessible via the public member 'modelDatDescription'.
    """
    
    def __init__(self):
        """
        This is the constructor, which is calling a function to initialize the public members
        'modelData' and 'modelDataDescription'.
        """
        self.__initData()
        #print('*** [' + self.__class__.__name__ + ']: initiated NeQuick-Fortran wrapper ...')
        pass
        
    def __del__(self):
        """
        This is the destructor.
        """
        pass
        
    def calcFromModel(self, data):        
        """
        NeQuick needs the following input:
        height [km], lat [deg], lon [deg], month, solarFlux, utcHour [double]
        """
        
        self.modelData['electronDensity'] = NeQuickIonosonde.nequickionosonde(data['height'],data['lat'],data['lon'],data['month'],data['f10p7'],data['utcHour'],data['doy'],data['year'],data['hour'],data['minute'])
        pass
        
    def __initData(self):
        """
        This function initializes some data structures and sets the data description
        for the requested output from NeQuickIonosonde.
        """
        self.modelData = {
            'electronDensity': 0.
        }
        self.modelDataDescription = {
            'electronDensity': 'Ne in [1/m**3]'
        }
        
        pass
        
    def help(self):
        """
        This function prints the original interface of the FORTRAN function NeQuickIonosonde from NeQuickIonosonde.for with 
        an explanation of the input and output parameters. Remark: Currently not implemented in NeQuickIonosonde.for.        
        print('*** [' + self.__class__.__name__ + ']: The following signature is wrapped with this interface:')
        print(NeQuickIonosonde.__doc__)
        print('*** [' + self.__class__.__name__ + ']: This wrapper returns a dictionary with the following output from PIM:')
        print(NeQuickIonosonde.__doc__)
        print(self.modelDataDescription)
        
        """ 
        print
        print('Help is currently not available!')

    def demo(self):
        """
        This function shows how to use the this wrapper class.
        """
        demoData = {        
        'height':18617.,
        'lat':-30.,
        'lon':-5,
        'month':1,
        'f10p7':77.4,
        'utcHour':0.0,
        'doy':12,
        'year':2011,
        'hour':0,
        'minute':0
        }
        
        print('\n\n*** [' + self.__class__.__name__ + ']: Running demo request for NeQuick with demo data:\n\n')
        print(demoData)
        self.calcFromModel(demoData)
        print('\n\n*** [' + self.__class__.__name__ + ']: *** Results:\n\n')
        print (self.modelData)
        print ('Expected Result using OI assimilation: 23452341.164194781 electrons/m**3') #with assimilation  

def main():
    
    wrapper = NeQuickIonosondeWrapper()
    wrapper.help()
    try:
		wrapper.demo()
    except:
		print '*** [' + self.__class__.__name__ + ']: Failure during calculation of electron density!'
if __name__ == "__main__":
    main()
