#!/usr/bin/env python
#-------------------------------------------------------------------------------
# Name:         NeQuickIonosondeWrapper.py
#
# Purpose:      This software wraps the compiled NeQuick Fortran model including assimilation of ionosonde key parameters hmF2 and foF2.
#
#               This software was developed to use the 3D electron density model NeQuick from python software.
#
# Needs:        python, f2py, gfortran, NeQuickIonosonde.so, 
#               assimilated foF2 and hmF2 maps (/data1/mink_da/3DReconstruction/SCM)
#
# Version:      $Id: PIMWrapper.py 7470 2013-09-25 12:07:01Z krie_ma $
#               $HeadURL: https://svn.dlr.de/GNSSsoft/trunk/NZ_Projects/IMPC/Tools/Python/NeQuickPythonInterface/NeQuick_PythonEnvironment/NeQuickIonosondeWrapper.py $
#
# Author:       Tatjana Gerzen
#               David Minkwitz
#               Martin Kriegel
#
# Created:      04.11.2013
# Copyright:    (c) Tatjana Gerzen & David Minkwitz & Martin Kriegel & 2013
# Mail:         Tatjana.Gerzen@dlr.de David.Minkwitz@dlr.de Martin.Kriegel@dlr.de 
# Licence:      DLR
#-------------------------------------------------------------------------------

import NeQuickIonosonde_gridProcessing, os
import numpy as np

class NeQuickIonosondeWrapper():
    """
    This class wraps the shared library NeQuick_2_0_2.so, which was compiled with gfortran and f2py
    during the build of the environment. The input parameters are wrapped in the dictionary data. The
    requested data from NeQuick_2_0_2 is accessible via the public member 'modelData', which is also
    a dictionary. A description for the 'modelData' is accessible via the public member 'modelDatDescription'.
    """
    
    def __init__(self):
        """
        This is the constructor, which is calling a function to initialize the public members
        'modelData' and 'modelDataDescription'.
        """
        self.__initData()
        print('*** [' + self.__class__.__name__ + ']: initiated NeQuick-Fortran wrapper ...')
        pass
        
    def __del__(self):
        """
        This is the destructor.
        """
        pass
        
    def calcFromModel(self, data):        
        """
        NeQuick needs the following input:
        height [km], lat [deg], lon [deg], month, solarFlux, utcHour [double]
        """
        #print '**********************'
        tmp = np.empty([73,145],dtype=np.float)
        tmp = NeQuickIonosonde_gridProcessing.gridcalculation(data['height'],data['minLat'],data['maxLat'],data['stepLat'],data['minLon'],data['maxLon'],data['stepLon'],data['year'],data['month'],data['doy'],data['hour'],data['minute'],data['utcHour'],data['f10p7'])
        #tmp = NeQuickIonosonde_gridProcessing.gridcalculation(data['height'],-90,90,2.5,-100,110,2.5,data['year'],data['month'],data['doy'],data['hour'],data['minute'],data['utcHour'],data['f10p7'])
        tmp = tmp[1]
        
        #print 'tmp.shape = ',tmp.shape
        #print 'tmp[0:84][4]: ',tmp[3][0:85]
        #self.modelData['electronDensity'] = tmp[:][0:85]
        
        #print "data:",data
        
        for latInd in range(0,73,1):
            self.modelData['electronDensity'][latInd] = tmp[latInd][0:85]
            
        
        #print "len(self.modelData['electronDensity'][0][:]): ",len(self.modelData['electronDensity'])
        #print 'shape: ',tmp[:][0:2].shape
        #print "electronDensity[0] = ",self.modelData['electronDensity'][0]
        #print self.modelData['electronDensity'].shape
        #np.savetxt('./ElectronDensityArray.dat',self.modelData['electronDensity'],fmt='%20.2f',delimiter='\t')
        #pass
        
    def __initData(self):
        """
        This function initializes some data structures and sets the data description
        for the requested output from NeQuickIonosonde.
        """
        
        
        self.modelData = {
            'electronDensity': np.empty([73,85],dtype=np.float)
        }
        self.modelDataDescription = {
            'electronDensity': 'Ne in [1/m**3]'
        }
        
        pass
        
    def help(self):
        """
        This function prints the original interface of the FORTRAN function NeQuickIonosonde from NeQuickIonosonde.for with 
        an explanation of the input and output parameters. Remark: Currently not implemented in NeQuickIonosonde.for.        
        print('*** [' + self.__class__.__name__ + ']: The following signature is wrapped with this interface:')
        print(NeQuickIonosonde.__doc__)
        print('*** [' + self.__class__.__name__ + ']: This wrapper returns a dictionary with the following output from PIM:')
        print(NeQuickIonosonde.__doc__)
        print(self.modelDataDescription)
        
        """ 
        print
        print('Help is currently not available!')

    def demo(self):
        """
        This function shows how to use the this wrapper class.
        """
        demoData = {
        'height':18617.,
        'lat':-30.,
        'lon':-5,
        'month':1,
        'f10p7':77.4,
        'utcHour':0.0,
        'doy':12,
        'year':2011,
        'hour':0,
        'minute':0,
        'minLat':-90,
        'maxLat':90.,
        'stepLat':2.5,
        'minLon':-100,
        'maxLon':110.,
        'stepLon':2.5
        }
        
        print('\n\n*** [' + self.__class__.__name__ + ']: Running demo request for NeQuick with demo data:\n\n')
        print(demoData)
        self.calcFromModel(demoData)
        print('\n\n*** [' + self.__class__.__name__ + ']: *** Results:\n\n')
        print (self.modelData)
        print ('Expected Result using ISCM IS RO assimilation: 23452341.5536 electrons/m**3') #with assimilation  

def main():
    
    wrapper = NeQuickIonosondeWrapper()
    wrapper.help()
  #  try:
    wrapper.demo()
 #   except:
#       print '*** [' + self.__class__.__name__ + ']: Failure during calculation of electron density!'
if __name__ == "__main__":
    main()
