#!/usr/bin/env python
#-------------------------------------------------------------------------------
# Name:         NeQuickWrapper.py
#
# Purpose:      This software wraps the compiled NeQuick Fortran model.
#
#               This software was developed to use the 3D electron density model NeQuick from python software.
#
# Needs:        python, f2py, gfortran, NeQuick_2_0_2.so
#
# Version:      $HeadURL: https://svn.dlr.de/GNSSsoft/trunk/NZ_Projects/IMPC/Tools/Python/NeQuickPythonInterface/NeQuick_PythonEnvironment/NeQuickWrapper.py $
#
# Author:       David Minkwitz & Martin Kriegel
#
# Created:      30.10.2013
# Copyright:    (c) David Minkwitz & Martin Kriegel 2013
# Mail:         Martin.Kriegel@dlr.de David.Minkwitz@dlr.de
# Licence:      DLR
#-------------------------------------------------------------------------------

import NeQuick_ECEF2LLH, os

class NeQuickWrapperECEF2LLH():
    """
    This class wraps the shared library NeQuick_2_0_2.so, which was compiled with gfortran and f2py
    during the build of the environment. The input parameters are wrapped in the dictionary data. The
    requested data from NeQuick_2_0_2 is accessible via the public member 'modelData', which is also
    a dictionary. A description for the 'modelData' is accessible via the public member 'modelDatDescription'.
    """
    
    def __init__(self):
        """
        This is the constructor, which is calling a function to initialize the public members
        'modelData' and 'modelDataDescription'.
        """
        self.__initData()
        print('*** [' + self.__class__.__name__ + ']: initiated NeQuick-Fortran wrapper ...')
        pass
        
    def __del__(self):
        """
        This is the destructor.
        """
        pass
        
    def calcFromModel(self, data):
        """
        This function wraps the FORTRAN function 'NeQuick_2_0_2.for'. Before requesting data from NeQuick_2_0_2
        the public members 'modelData' and 'modelDataDescription' are initialized. After that
        the NeQuick function from NeQuick_2_0_2.for (included in NeQuick_2_0_2.so) is called with the given input data dictionary.
        Finally the output data is assigned to the public member 'modelData'.
        """
        self.__initData()
        
        """
        NeQuick needs the following input:
        height [km], lat [deg], lon [deg], month, solarFlux, utcHour [double]
        """
        
        self.modelData['electronDensity'] = NeQuick_ECEF2LLH.nequick(data['x'],data['y'],data['z'],data['month'],data['f10p7'],data['utcHour'])
        pass
        
    def __initData(self):
        """
        This function initializes some data structures and sets the data description
        for the requested output from NeQuick.
        """
        self.modelData = {
            'electronDensity': 0.
        }
        self.modelDataDescription = {
            'electronDensity': 'Ne in [1/m**3]'
        }
        
        pass
        
    def help(self):
        print('Help is currently not available!')
        
    def demo(self):
        """
        This function shows how to use the this wrapper class.
        'height':400.,
        'lat':50.,
        'lon':10.,
        'month':1,
        'f10p7':130.,
        'utcHour':14.5
        
        """
        demoData = {
        'x':4298665.293981508,
        'y':757970.672364025,
        'z':5169206.814905126,
        'month':1,
        'f10p7':130.,
        'utcHour':14.5
        }
        
        print('\n\n*** [' + self.__class__.__name__ + ']: Running demo request for NeQuick with demo data:\n\n')
        print(demoData)
        self.calcFromModel(demoData)
        print('\n\n*** [' + self.__class__.__name__ + ']: *** Results:\n\n')
        print (self.modelData)
        print ('Expected Result: 388680864447.93378 electrons/m**3')
        
    def demo_vs2(self):
        """
        This function shows how to use the this wrapper class.
        """
        demoData = {
        'height':480.,
        'lat':51.,
        'lon':-1.3,
        'month':1,
        'f10p7':82.,
        'utcHour':7.0
        }
   
        
        print('\n\n*** [' + self.__class__.__name__ + ']: Running demo request for NeQuick with demo data:\n\n')
        print(demoData)
        self.calcFromModel(demoData)
        print('\n\n*** [' + self.__class__.__name__ + ']: *** Results:\n\n')
        print (self.modelData)
        print ('Expected Result: 24047509229.6297 electrons/m**3') #with assimilation:  30177434107.5882 

def main():
    
    wrapper = NeQuickWrapper()
    #wrapper.help()
   # wrapper.demo()
    wrapper.demo_vs2()
    
if __name__ == "__main__":
    main()
