#!/usr/bin/env python
#-------------------------------------------------------------------------------
# Name:         slQuNewWrapper.py
#
# Purpose:      This software wraps the compiled NeQuick Fortran model for the calculation of slant TEC between two given locations.
#
#               This software was developed to use the 3D electron density model NeQuick from python software.
#
# Needs:        python, f2py, gfortran, slQuNew.so
#
# Version:      $HeadURL: https://svn.dlr.de/GNSSsoft/trunk/NZ_Projects/IMPC/Tools/Python/NeQuickPythonInterface/NeQuick_PythonEnvironment/slQuNewWrapper.py $
#
# Author:       David Minkwitz
#
# Created:      04.11.2013
# Copyright:    (c) David Minkwitz 2013
# Mail:         David.Minkwitz@dlr.de 
# Licence:      DLR
#-------------------------------------------------------------------------------

import slQuNew, os

class slQuNewWrapper():
    """
    This class wraps the shared library NeQuick_2_0_2.so, which was compiled with gfortran and f2py
    during the build of the environment. The input parameters are wrapped in the dictionary data. The
    requested data from NeQuick_2_0_2 is accessible via the public member 'modelData', which is also
    a dictionary. A description for the 'modelData' is accessible via the public member 'modelDatDescription'.
    """
    
    def __init__(self):
        """
        This is the constructor, which is calling a function to initialize the public members
        'modelData' and 'modelDataDescription'.
        """
        self.__initData()
        print('*** [' + self.__class__.__name__ + ']: initiated NeQuick-Fortran wrapper ...')
        pass
        
    def __del__(self):
        """
        This is the destructor.
        """
        pass
        
    def calcFromModel(self, data):
        """
        This function wraps the FORTRAN function 'NeQuick_2_0_2.for'. Before requesting data from NeQuick_2_0_2
        the public members 'modelData' and 'modelDataDescription' are initialized. After that
        the NeQuick function from NeQuick_2_0_2.for (included in NeQuick_2_0_2.so) is called with the given input data dictionary.
        Finally the output data is assigned to the public member 'modelData'.
        """
        self.__initData()
        
        """
        slQuNew needs the following input:
        height [km], lat [deg], lon [deg], month, solarFlux, utcHour [double]
        'lat1':60.,
        'lon1':10.,
        'hgt1':0.,
        'lat2':60.,
        'lon2':10.,
        'hgt2':22000.,
        'day':15,
        'month':1,
        'year':2011,
        'utcHour':14.0,
        'f10p7':130.,
        """
        
        self.modelData['sTEC'] = slQuNew.slqunew(data['lat1'],data['lon1'],data['hgt1'],data['lat2'],data['lon2'],data['hgt2'],data['day'],data['month'],data['year'],data['utcHour'],data['f10p7'])
        self.modelData['sTEC'] = self.modelData['sTEC']/10.
        pass
        
    def __initData(self):
        """
        This function initializes some data structures and sets the data description
        for the requested output from slQuNew.
        """
        self.modelData = {
            'sTEC': 0.
        }
        self.modelDataDescription = {
            'sTEC': 'sTEC in TECU'
        }
        
        pass
        
    def help(self):
        print
        print(self.modelDataDescription)
        print('further help is currently not available!')

    def demo(self):
        """
        This function shows how to use the this wrapper class.
        position with lower height is named pos1(lat1,lon1,hgt1), latitude/longitude in degree, height in km
        """
        demoData = {
        'lat1':-85.,
        'lon1':-180.,
        'hgt1':0.,
        'lat2':-85.,
        'lon2':-180.,
        'hgt2':760.,
        'day':1,
        'month':1,
        'year':2011,
        'utcHour':14.0,
        'f10p7':193.,
        }
        
        print('\n\n*** [' + self.__class__.__name__ + ']: Running demo request for NeQuick with demo data:\n\n')
        print(demoData)
        self.calcFromModel(demoData)
        print('\n\n*** [' + self.__class__.__name__ + ']: *** Results:\n\n')
        print (self.modelData)
        print ('Expected Result: ',23.736646/2.,' TECU') #with assimilation  

def main():
    
    wrapper = slQuNewWrapper()
    wrapper.help()
    wrapper.demo()
    
if __name__ == "__main__":
    main()
