import numpy as np
import sys
import VariogramInR
import math
import scipy.optimize as optimize
import imp
import matplotlib.pyplot as plt
import MeasurementStationsCoKriging as StationsCK
import config

class OrdinaryCoKriging(object):

    def __init__(self, stationshmf2, stationsnmf2, variogrammodell, outputfolder, model, doValidation, f107, hour, doy, month, outputfolderLog, logfilename, resolution=2.5):
        '''
        Konstructor.
        :param stations: MeasurementStations object
        :param variogrammodell: "exp", "sph" or "gau"
        :param Outputfolder: path, where the pictures sould be stored
        :param model: "NPDM", "NPHM" or "NeQuick"
        :param doValidation: Boolean value, if the validation should be done or not
        :param f107: List of F10.7 values
        :param hour: hour of the day
        :param doy: Day of the year
        :param month: month
        :param outputfolderLog: Path for the log files
        :param logfilename: name for the log file
        :param resolution: resolution for the estimation map. Standard 2.5
        :return:
        '''
        self.StationshmF2 = stationshmf2
        self.StationsNmF2 = stationsnmf2
        self.EarthRadius = 6378     #in km
        self.Nugget = 0
        self.Sill = 1
        self.Range = 3000
        self.minLat = -90
        self.maxLat = 90
        self.minLon = -180
        self.maxLon = 180
        self.Resolution = resolution
        self.Outputfolder = outputfolder
        self.Model = model
        self.VarMod = variogrammodell
        self.DoValidation = doValidation
        self.LogFileName = logfilename
        self.F107 = f107
        self.Hour = hour
        self.Month = month
        self.Doy = doy
        self.OutputfolderLog = outputfolderLog
        self.Filterdistance = 15000

    def coKriging(self):
        '''
        This function calculates the prediction value and the prediction error.
        :return:
        '''

        lats = []
        lons = []
        if self.Resolution == 1:
            lats = range(self.minLat, self.maxLat, self.Resolution)
            lons = range(self.minLon, self.maxLon, self.Resolution)
        else:
            i = self.maxLat
            while i >= self.minLat:
                lats.append(i)
                i -= self.Resolution
            i = self.minLon
            while i <= self.maxLon:
                lons.append(i)
                i += self.Resolution

        if self.DoValidation:
            self.EqualStations.removeRefStations(config.refStations)
            self.StationshmF2 = self.StationshmF2.removeRefStations()
            self.StationsNmF2 = self.StationsNmF2.removeRefStations()

        matrixhmf2 = self.getMatrixhmF2()
        matrixnmF2 = self.getMatrixNmF2()
        matrixNmf2hmf2 = self.getMatrixNmF2hmF2()

        inverseGammas = self.getInverseMatrix(matrixhmf2, matrixnmF2, matrixNmf2hmf2)

        #validation matrix
        valMatrix = np.zeros((len(lats), len(lons)))
        predErr = np.zeros((len(lats), len(lons)))
        counterLats = 0

        for i in lats:
            counterLons = 0
            for j in lons:
                #This is the vector on the right hand site of the equation (Wackernagel Page 80 Formula 11.6
                distvector = self.spatialDistanceVector(self.Lats, self.Lons, i, j)
                gg = np.matrix(np.vstack([self.getGammasVector(distvector), [1, ]]))
                weights = inverseGammas * gg
                if not 0.9 < np.sum(weights[:-1]) < 1.1:
                    raise AssertionError("Sum of the weights is != 1")
                predErr[counterLats][counterLons] = np.sqrt(np.abs(weights[len(weights) - 1] + np.sum(gg[:-1].A1 * weights[:-1].A1)))

                if i == 0: i = 0.000000000001               #workaround; else a warning would appear in the npdm model reader
                valMatrix[counterLats][counterLons] = self.getModelValue(i, j) + (self.DiffModelValues * weights[:-1])

                counterLons += 1
            counterLats += 1

        if self.DoValidation:
            refStations = self.Stations.getRefStations()
            stationIDs = refStations.getAllStationID()

            for k in range(0, refStations.getCount()):
                distvector = self.spatialDistanceVector(self.Lats, self.Lons, refStations.getLat(k), refStations.getLon(k))
                if config.isotropyAnisotropy == "anisotropy":
                    gg1 = np.matrix(np.vstack([self.getGammasVectorAnisotropy(distvector), [1, ]]))
                else:
                    gg1 = np.matrix(np.vstack([self.getGammasVector(distvector), [1, ]]))
                weights = inverseGammas * gg1
                if not 0.9 < np.sum(weights[:-1]) < 1.1:
                    print self.Doy, self.VarMod, self.Layer, self.Model
                    raise AssertionError("Sum of the weights is != 1")
                estimatedValue = float(self.getModelValue(refStations.getLat(k), refStations.getLon(k)) + (self.DiffModelValues * weights[:-1]))
                MeasurementValue = self.Stations.getMeasurementFromID(stationIDs[k])
                residualsRefRel = ((MeasurementValue - estimatedValue) / MeasurementValue) * 100
                self.writeResiduals4Validation(residualsRefRel, stationIDs[k], self.OutputfolderLog + self.Layer + "_" + self.Model + self.VarMod + "ResidualsRefRel.dat")
                residualsRefAbs = MeasurementValue - estimatedValue
                self.writeResiduals4Validation(residualsRefAbs, stationIDs[k], self.OutputfolderLog + self.Layer + "_" + self.Model + self.VarMod + "ResidualsRefAbs.dat")

        return valMatrix, predErr

    def getInverseMatrix(self, hmf2, nmf2, hmf2nmf2):

        temp_links = np.vstack((hmf2, hmf2nmf2))

        temp_rechts = np.vstack((hmf2nmf2, nmf2))
        matrix = self.stickMatrixTogether(temp_links, temp_rechts)

        np.savetxt("final", matrix)

    def stickMatrixTogether(self, matrix1, matrix2):
        '''
        This funtion sticks two matrices side by side togther and adds teh Langrange paramter. [[m1][m2]]
        :param matrix1: first matrix (left hand site)
        :param matrix2: second matrix (right hand site)
        :return: matrix
        '''

        newmatrix = np.zeros((len(matrix1) + 2, len(matrix1) + 2))           #+2 for Langrange parameter
        columnsmatrix1 = len(matrix1[1])
        print columnsmatrix1, len(newmatrix), len(newmatrix[0])
        for i in range(len(matrix1)):
            for j in range(len(matrix1[0])):
                newmatrix[i][j] = matrix1[i][j]
                newmatrix[i][j + columnsmatrix1] = matrix2[i][j]



        #langrange parameter
        ## for i in range(len())
        return newmatrix

    def getMatrixhmF2(self):
        '''
        This function returns the inverse of the kriging matrix.
        :return: inverse distance matrix
        '''

        distmatrix = self.spatialDistanceMatrix(self.EqualStations.getAllLat(), self.EqualStations.getAllLon())

        return self.getGammasMatrix(distmatrix)

    def getMatrixNmF2(self):
        '''
        This function returns the inverse of the kriging matrix.
        :return: inverse distance matrix
        '''

        distmatrix = self.spatialDistanceMatrix(self.EqualStations.getAllLat(), self.EqualStations.getAllLon())

        return self.getGammasMatrix(distmatrix)

    def getMatrixNmF2hmF2(self):
        '''
        This function returns the inverse of the kriging matrix.
        :return: inverse distance matrix
        '''

        distmatrix = self.spatialDistanceMatrix(self.EqualStations.getAllLat(), self.EqualStations.getAllLon())

        return self.getGammasMatrix(distmatrix)

    def spatialDistanceMatrix (self, lats, lons):
        '''
        This function calculates the distance matrix.
        :param coordsRow: coordinates of every station
        :param coordsColumn: coordinates of every station (coordsColumn must be equal with coordsRow)
        :return: distance matrix
        '''
        distMatrix = np.zeros((len(lats), len(lons)))
        for i in range(len(lats)):
            for j in range(len(lons)):
                if i != j: distMatrix[i][j] = self.calcDistSpherical(lats[i], lons[i], lats[j], lons[j])
        return distMatrix

    def getGammasMatrix(self, distmatrix):
        '''
        This function calculates the gamma matrix. The gammas are the result of the variogram function.
        See also Multivariate Geostatistics from Wackernagel Page 80 Equation 11.6 left hand site of the equation
        :param distmatrix: distance matrix
        :return: matrix of gammas
        '''
        if self.VarMod == "exp":
            for i in range(len(distmatrix)):
                for j in range(len(distmatrix)):
                    distmatrix[i][j] = self.exponential(distmatrix[i][j])
        elif self.VarMod == "sph":
            for i in range(len(distmatrix)):
                for j in range(len(distmatrix)):
                    distmatrix[i][j] = self.spherical(distmatrix[i][j])
        elif self.VarMod == "gau":
            for i in range(len(distmatrix)):
                for j in range(len(distmatrix)):
                    distmatrix[i][j] = self.gaussian(distmatrix[i][j])
        elif self.VarMod == "pow":
            for i in range(len(distmatrix)):
                for j in range(len(distmatrix)):
                    distmatrix[i][j] = self.power(distmatrix[i][j])
        else:
            raise AssertionError("Variogramodel not found! Choose between 'exp' for exponential, 'sph' for spherical, 'gau' for gaussian or 'pow' for power!")

        return distmatrix

    def calcVariogramParams(self, lag):
        '''
        After initialising the Object, this is the first method to call, because the variogram parameter must be estimated,
        before the krige prediction starts. Additionally this function calls the plot functions for the variograms
        :param lag: lag distance
        :return:
        '''
        self.Lag = lag

        distances, semivar = self.findDistancesAndSemivariances()

        distances, semivar = self.filterDistSemivar(distances, semivar)

        findModul = False
        try:
            imp.find_module('rpy2')
            findModul = True
        except:
            self.writeInLog("Warning: Could not find the modul rpy2. Variogram calculation will be done in Python with Least Square. Look up the manual for more information.")
            print "Warning: Could not find the modul rpy2. Variogram calculation will be done in Python with Least Square. Look up the manual for more information."
            findModul = False

        ###############################################################
        #####################Estimation in R###########################
        ###############################################################
        if findModul:
            try:
                varioInR = VariogramInR.VariogramInR(self.Lag, self.Filterdistance, self.VarMod, "NmF2", self.Outputfolder, self.OutputfolderLog, self.Model)
                estimated_varioparam = varioInR.manageEstimationInR(distances, semivar, "Isotropy")
                self.Sill, self.Range, self.Nugget = estimated_varioparam
                #print "In R (" + self.VarMod + ") (sill, range, nugget): ", estimated_varioparam
            except:
                print "Error message: " + str(sys.exc_info())
                self.manageEstimationLeastSquare(distances, semivar, "Isotropy")
        ###############################################################
        #####################Least Squares#############################
        ###############################################################
        else:
            estimated_varioparam = self.manageEstimationLeastSquare(distances, semivar, "Isotropy")
            #print "In Python (" + self.VarMod + ") (sill, range, nugget): ", estimated_varioparam

    def findDistancesAndSemivariances(self):
        '''
        This function calculates the distances and semivariances for every station, which has a hmF2 and NmF2 value.
        :return: distances and semivariances in two lists
        '''
        distances = []
        semivar = []

        self.EqualStations = StationsCK.MeasurementStationsCoKriging()
        for i in range(0, self.StationsNmF2.getCount()):
            if self.StationsNmF2.getStationID(i) in self.StationshmF2.getAllStationID():
                id = self.StationsNmF2.getStationID(i)
                lat = self.StationsNmF2.getLat(i)
                lon = self.StationsNmF2.getLon(i)
                pos = self.StationshmF2.getPosOfStation(id)
                hmf2 = self.StationshmF2.getMeasurement(pos)
                nmf2 = self.StationsNmF2.getMeasurement(i)
                self.EqualStations.addStation(id, lat, lon, hmf2, nmf2)

        for i in range(0, self.EqualStations.getCount()):
            for j in range(0, self.EqualStations.getCount()):
                lat1 = self.EqualStations.getLat(i)
                lon1 = self.EqualStations.getLon(i)
                lat2 = self.EqualStations.getLat(j)
                lon2 = self.EqualStations.getLon(j)
                distances.append(self.calcDistSpherical(lat1, lon1, lat2, lon2))
                hmf2_1 = self.EqualStations.gethmF2(i)
                hmf2_2 = self.EqualStations.gethmF2(j)
                nmf2_1 = self.EqualStations.getNmF2(i)
                nmf2_2 = self.EqualStations.getNmF2(j)
                semivar.append(np.abs(((hmf2_2 - hmf2_1) * (nmf2_2 - nmf2_1))) / 2)

        return distances, semivar

    def gaussian(self, h):
        '''
        This function return the value for the gaussian variogram
        :param h: Distance between two stations
        :return: the variogram value for h
        '''
        return (self.Sill * (1 - np.exp((-3 * h**2) / self.Range**2)))

    def spherical(self, h):
        '''
        This function return the value for the spherical variogram
        :param h: Distance between two stations
        :return: the variogram value for h
        '''
        if h < self.Range: return (self.Sill * (1.5 * (h / self.Range) - 0.5 * ((h/self.Range)**3)))
        else: return self.Sill

    def exponential(self, h):
        '''
        This function return the value for the exponential variogram
        :param h: Distance between two stations
        :return: the variogram value for h
        '''
        return (self.Sill * (1 - np.exp((-3 * h) / self.Range)))

    def power(self, h):
        '''
        This function return the value for the power variogram
        :param h: Distance between two stations
        :return: the variogram value for h
        '''
        return (self.Sill * ((h / self.Range) ** self.Nugget))          #IT IS NOT THE NUGGET IT IS ONLY A NAME FOR ALPHA

    def findMinMax(self, matrix):
        '''
        This functions finds the minimum and maximum value in a matrix
        :param matrix: Matrix
        :return: minumum value, maximum value
        '''
        minval = matrix[0][0]
        maxval = matrix[0][0]

        for i in range(len(matrix)):
            for j in range(len(matrix[0])):
                if minval > matrix[i][j]: minval = matrix[i][j]
                elif maxval < matrix[i][j]: maxval = matrix[i][j]
        return minval, maxval

    def manageEstimationLeastSquare(self, distances, semivar, iso):
        '''
        This function manages the calculation of the variogram parameter with the Least Square Method
        :param distances: list of all distances
        :param semivar: list of semivariances
        :param iso: identifier for anisotropy or isotropy
        :return: variogramparameter (sill, range, nugget) only to write on console
        '''

        distances = np.asarray(distances)
        semivar = np.asarray(semivar)

        sortind = np.argsort(distances)
        sortdist = distances[sortind]
        sortsvar = semivar[sortind]

        index = sortdist.searchsorted(np.linspace(0, np.max(distances), self.Lag+1))

        distances = [sortdist[index[i-1]:index[i]].mean() for i in range(1, len(index))]
        semivar = [sortsvar[index[i-1]:index[i]].mean() for i in range(1, len(index))]
        tempdis = []
        tempvar = []
        for i in range(len(distances)):
            if not math.isnan(distances[i]) and not math.isnan(semivar[i]):
                tempdis.append(distances[i])
                tempvar.append(semivar[i])

        semivar = tempvar
        distances = tempdis

        varrange = 5000
        varsill = (0.9 * max(self.DiffModelValues)) ** 2
        #varsill = 2e22
        varnugget = 0
        param = (varsill, varrange, varnugget)
        estimated_varioparam = optimize.leastsq(self.leastSquareFit, param, args=(semivar, distances), full_output=0)
        if estimated_varioparam:
            self.Sill, self.Range, self.Nugget = estimated_varioparam[0]
            self.Nugget = 0
        else: raise AssertionError("Bad fitting calculation!")

        if self.VarMod == "exp": modelname = "Exponential Variogram"
        elif self.VarMod == "sph": modelname = "Spherical Variogram"
        elif self.VarMod == "gau": modelname = "Gaussian Variogram"
        elif self.VarMod == "pow": modelname = "Power Variogram"
        else:
            raise AssertionError("Variogramodel not found! Choose between 'exp' for exponential, 'sph' for spherical, 'gau' for gaussian or 'pow' for Power!")

        varioparams = "SemiVariogram %s %s %s\n Sill:%.1f Range:%.1f Nugget:%.1f"%(self.Model, self.Layer, modelname, self.Sill, self.Range, self.Nugget)
        self.plotVariogram(distances, semivar, varioparams, iso)

        return estimated_varioparam

    def plotVariogram(self, distances, semivar, varioparams, iso):
        '''
        This function plots the variogramcloud, the experimental variogram and the variogramcurve in both variograms.
        :param distances: list of distances
        :param semivar: list of semivariances
        :param varioparams: variogram parameter (param = [sill, range, nugget])
        :return:
        '''

        h = np.arange(0, np.max(distances), 0.1)
        varcurve = []
        for value in h:
            if self.VarMod == "exp": varcurve.append(self.exponential(value))
            elif self.VarMod == "sph": varcurve.append(self.spherical(value))
            elif self.VarMod == "gau": varcurve.append(self.gaussian(value))
            elif self.VarMod == "pow": varcurve.append(self.power(value))
            else:
                raise AssertionError("Variogramodel not found! Choose between 'exp' for exponential, 'sph' for spherical, 'gau' for gaussian or 'pow' for power!")

        plt.figure()
        plt.plot(h, varcurve, 'r-')
        plt.plot(distances, semivar, 'o')
        plt.title(varioparams)
        plt.savefig(self.Outputfolder + self.Layer + "_" + self.Model + "_" + self.VarMod + "_" + "Variogramcloud" + iso + ".png")
        plt.close()
        #self.writeInListofResults(self.Layer + "_" + self.Model + "_" + self.VarMod + "_" + "Variogramcloud" + iso + ".png")

        lagIntervall = []
        plotIntervall = []
        lagTemp = self.Lag / 2
        i = 0
        countLag = 0
        lagIntervall.append(0)
        plotIntervall.append(0.25 * self.Lag)
        while countLag < self.Filterdistance:
            lagIntervall.append(lagTemp + self.Lag * i)
            if i != 0: plotIntervall.append(lagTemp * i * 2)
            i += 1
            countLag += self.Lag

        mean, counter = self.getMeanInLag(zip(distances, semivar), lagIntervall)
        plt.figure()
        plt.plot(h, varcurve, 'r-')
        plt.scatter(plotIntervall, mean, counter)
        plt.legend()
        plt.xlabel("distance")
        plt.ylabel("semivariance")
        plt.title(varioparams)
        plt.savefig(self.Outputfolder + self.Layer + "_" + self.Model + "_" + self.VarMod + "_" + "Experimental" + iso + ".png")
        plt.close()
        #self.writeInListofResults(self.Layer + "_" + self.Model + "_" + self.VarMod + "_" + "Experimental" + iso + ".png")

    def calcmean(self, list):
        '''
        This function calculates the mean of the list. If the list is empty, zero will be returned
        :param list: list of values
        :return: mean of the list, zero if empty
        '''
        if not list: return 0
        return np.mean(list)

    def getMeanInLag(self, distVal, lagIntervall):
        '''
        This function calculates the mean in a lag distance. If there is no value in a lag, zero will be added to the list
        :param distVal: zipped list of distances and semivariances (zip(distances, semivariances))
        :param lagIntervall: list of the lagintervalls
        :return: list of all means for every lag distance and a list, which stored the count of the values in every lag distance
        '''
        listOfMeans = []
        counter = []
        for i in range(len(lagIntervall)-1):
            listOfVal = []
            for j in range(len(distVal)):
                if lagIntervall[i] <= distVal[j][0] < lagIntervall[i+1]:
                    listOfVal.append(distVal[j][1])
            listOfMeans.append(self.calcmean(listOfVal))
            counter.append(len(listOfVal))

        return listOfMeans, counter

    def writeResiduals4Validation(self, residual, stationid, filename):
        '''
        This functions writes the residuals of a station in a file.
        :param residual: list of residuals for every station
        :param stationid: station id
        :param filename: Filename
        :return:
        '''
        fopen = open(filename, 'r')
        allLines = []
        for line in fopen:
            if stationid in line:
                line = line.replace("\n", "")
                line = line + ";" + str(residual) + "\n"
            allLines.append(line)
        fopen.close()

        fopen = open(filename, 'w')
        for i in range(len(allLines)):
            fopen.write(allLines[i])
        fopen.close()

    def writeInLog(self, message):
        '''
        This function writes all massages in a log file.
        :param message: message to write
        :return:
        '''
        fobj = open(self.OutputfolderLog + self.LogFileName, "a")
        fobj.writelines(message + "\n")
        fobj.close()

    def filterDistSemivar(self, distances, semivar):
        '''
        This function is used to remove the distances in the variogram, which are greater then 10.000km. The reason for this is,
        to get a better variogram estimation
        :param distances: list of all distances
        :param semivar: list of all semivariances
        :return: return a new list for all distances and semivariances
        '''
        filteredDist = []
        filteredSemivar = []
        for i in range(len(distances)):
            if (distances[i] < self.Filterdistance):
                filteredDist.append(distances[i])
                filteredSemivar.append(semivar[i])

        return filteredDist, filteredSemivar

    def calcDistSpherical(self, lat1, lon1, lat2, lon2):
        '''
        This function calculates the spherical distance between two points. Haversine formula used.
        :param lat1: latitude of origin
        :param lon1: longitude of origin
        :param lat2: latitude of destination
        :param lon2: longitude of destination
        :return: spherical distance of the two points
        '''
        if lat1 == lat2 and lon2 == lon1: return 0.0
        dlat = math.radians(lat2-lat1) / 2
        dlon = math.radians(lon2-lon1) / 2
        a = math.sin(dlat) * math.sin(dlat) + math.cos(math.radians(lat1)) * math.cos(math.radians(lat2)) * math.sin(dlon) * math.sin(dlon)
        c = 2 * math.atan2(math.sqrt(a), math.sqrt(1-a))
        return self.EarthRadius * c

    def leastSquareFit(self, param, semivar, distances):
        '''
        This function estimates the variogram parameter sill, range and nugget with the use of the least square method.
        :param param: initialized variogram parameter (param = [sill, range, nugget])
        :param semivar: list of semivariances
        :param distances: list of distances
        :return: estimated parameter in the structur [sill, range, nugget]
        '''
        varFuncRes = []

        self.Sill, self.Range, self.Nugget = param
        if self.Nugget < 0: self.Nugget = 0
        if self.Nugget > self.Sill: self.Nugget = self.Sill

        for i in range(len(distances)):
            if self.VarMod == "exp":
                varFuncRes.append(self.exponential(distances[i]))
            elif self.VarMod == "sph":
                varFuncRes.append(self.spherical(distances[i]))
            elif self.VarMod == "gau":
                varFuncRes.append(self.gaussian(distances[i]))
            elif self.VarMod == "pow":
                varFuncRes.append(self.power(distances[i]))
            else:
                raise AssertionError("Variogramodel not found! Choose between 'exp' for exponential, 'sph' for spherical, 'gau' for gaussian or 'pow' for power!")

        leastSq = np.asarray(semivar) - np.asarray(varFuncRes)
        return leastSq