# coding=utf-8
import numpy as np
import math
import matplotlib.pyplot as plt
from mpl_toolkits.basemap import Basemap
import scipy.optimize as optimize
import matplotlib.colors as mc
import NeQuick.NeQuick_PythonEnvironment.NeQuickWrapperVectorInput as NeQuick
import bin.reconstruct.ModelNPDM_NPHM.NEDMv1 as NEDM
import VariogramInR as VariogramInR
import imp
import sys
import config

class OrdinaryKriging(object):
    '''
    This class contains all function, which are needed to calculate variograms and ordinary kriging. Additionally the functions save all images as
    .png file
    '''
    def __init__(self, stations, variogrammodell, outputfolder, model, doValidation, f107, hour, doy, month, outputfolderLog, logfilename, resolution=2.5):
        '''
        Konstructor.
        :param stations: MeasurementStations object
        :param variogrammodell: "exp", "sph" or "gau"
        :param Outputfolder: path, where the pictures sould be stored
        :param model: "NPDM", "NPHM" or "NeQuick"
        :param doValidation: Boolean value, if the validation should be done or not
        :param f107: List of F10.7 values
        :param hour: hour of the day
        :param doy: Day of the year
        :param month: month
        :param outputfolderLog: Path for the log files
        :param logfilename: name for the log file
        :param resolution: resolution for the estimation map. Standard 2.5
        :return:
        '''
        self.Stations = stations
        self.Lats = stations.getAllLat()
        self.Lons = stations.getAllLon()
        self.DiffModelValues = stations.getResiduals()
        self.MeasurementValues = stations.getAllMeasurement()
        self.EarthRadius = 6378     #in km
        self.Nugget = 0
        self.Sill = 1
        self.Range = 3000
        self.minLat = -90
        self.maxLat = 90
        self.minLon = -180
        self.maxLon = 180
        self.Resolution = resolution
        self.Outputfolder = outputfolder
        self.Model = model
        self.Layer = stations.getLayer()
        self.VarMod = variogrammodell
        self.DoValidation = doValidation
        self.LogFileName = logfilename
        self.F107 = f107
        self.Hour = hour
        self.Month = month
        self.Doy = doy
        self.OutputfolderLog = outputfolderLog
        self.Filterdistance = 15000

    def calcVariogramParams(self, lag):
        '''
        After initialising the Object, this is the first method to call, because the variogram parameter must be estimated,
        before the krige prediction starts. Additionally this function calls the plot functions for the variograms
        :param lag: lag distance
        :return:
        '''
        self.Lag = lag
        distances = []
        semivar = []
        angles = []

        for i in range(len(self.Lats)):
            for j in range(len(self.Lons)):
                semivar.append((self.DiffModelValues[i] - self.DiffModelValues[j])**2 / 2)
                distances.append(self.calcDistSpherical(self.Lats[i], self.Lons[i], self.Lats[j], self.Lons[j]))
                angles.append(self.calcAngle(self.Lats[i], self.Lons[i], self.Lats[j], self.Lons[j]))


        ###############################################################
        ###########Estimation in R directional Variograms##############
        ###############################################################
        if config.isotropyAnisotropy == "anisotropy":
            self.manageEstimationAnisotropy(distances, semivar, angles)
        ###############################################################
        else:
            distances, semivar = self.filterDistSemivar(distances, semivar)

            findModul = False
            try:
                imp.find_module('rpy2')
                findModul = True
            except:
                self.writeInLog("Warning: Could not find the modul rpy2. Variogram calculation will be done in Python with Least Square. Look up the manual for more information.")
                print "Warning: Could not find the modul rpy2. Variogram calculation will be done in Python with Least Square. Look up the manual for more information."
                findModul = False

            ###############################################################
            #####################Estimation in R###########################
            ###############################################################
            if findModul:
                try:
                    varioInR = VariogramInR.VariogramInR(self.Lag, self.Filterdistance, self.VarMod, self.Layer, self.Outputfolder, self.OutputfolderLog, self.Model)
                    estimated_varioparam = varioInR.manageEstimationInR(distances, semivar, "Isotropy")
                    self.Sill, self.Range, self.Nugget = estimated_varioparam
                    print "In R (" + self.VarMod + ") (sill, range, nugget): ", estimated_varioparam
                except:
                    print "Error message: " + str(sys.exc_info())
                    self.manageEstimationLeastSquare(distances, semivar, "Isotropy")
            ###############################################################
            #####################Least Squares#############################
            ###############################################################
            else:
                estimated_varioparam = self.manageEstimationLeastSquare(distances, semivar, "Isotropy")

            if self.VarMod == "pow": self.writeInLog("Variogram parameter (isotrop) (" + self.VarMod + ") (b, alpha): " + str(self.Sill) + str(self.Nugget))
            else: self.writeInLog("Variogram parameter (isotrop) (" + self.VarMod + ") (sill, range, nugget): " + str(self.Sill) + str(self.Range) + str(self.Nugget))

    def myKriging(self):
        '''
        This function calculates the prediction value and the prediction error.
        :return:
        '''

        lats = []
        lons = []
        if self.Resolution == 1:
            lats = range(self.minLat, self.maxLat, self.Resolution)
            lons = range(self.minLon, self.maxLon, self.Resolution)
        else:
            i = self.maxLat
            while i >= self.minLat:
                lats.append(i)
                i -= self.Resolution
            i = self.minLon
            while i <= self.maxLon:
                lons.append(i)
                i += self.Resolution

        if self.DoValidation:
            without_refstations = self.Stations.removeRefStations()
            self.Lats = without_refstations.getAllLat()
            self.Lons = without_refstations.getAllLon()
            self.MeasurementValues = without_refstations.getAllMeasurement()
            self.DiffModelValues = without_refstations.getResiduals()

        #This is the matrix on the left hand site of the equation (Wackernagel Page 80 Formula 11.6
        transCoords = np.asarray([self.Lats, self.Lons]).T
        inverseGammas = self.getInverse(transCoords)

        #validation matrix
        valMatrix = np.zeros((len(lats), len(lons)))
        predErr = np.zeros((len(lats), len(lons)))
        counterLats = 0

        for i in lats:
            counterLons = 0
            for j in lons:
                #This is the vector on the right hand site of the equation (Wackernagel Page 80 Formula 11.6
                distvector = self.spatialDistanceVector(self.Lats, self.Lons, i, j)
                if config.isotropyAnisotropy == "anisotropy":
                    gg = np.matrix(np.vstack([self.getGammasVectorAnisotropy(distvector), [1, ]]))
                else:
                    gg = np.matrix(np.vstack([self.getGammasVector(distvector), [1, ]]))
                weights = inverseGammas * gg
                if not 0.9 < np.sum(weights[:-1]) < 1.1:
                    raise AssertionError("Sum of the weights is != 1")
                predErr[counterLats][counterLons] = np.sqrt(np.abs(weights[len(weights) - 1] + np.sum(gg[:-1].A1 * weights[:-1].A1)))

                if i == 0: i = 0.000000000001               #workaround; else a warning would appear in the npdm model reader
                valMatrix[counterLats][counterLons] = self.getModelValue(i, j) + (self.DiffModelValues * weights[:-1])

                counterLons += 1
            counterLats += 1

        if self.DoValidation:
            refStations = self.Stations.getRefStations()
            stationIDs = refStations.getAllStationID()

            for k in range(0, refStations.getCount()):
                distvector = self.spatialDistanceVector(self.Lats, self.Lons, refStations.getLat(k), refStations.getLon(k))
                if config.isotropyAnisotropy == "anisotropy":
                    gg1 = np.matrix(np.vstack([self.getGammasVectorAnisotropy(distvector), [1, ]]))
                else:
                    gg1 = np.matrix(np.vstack([self.getGammasVector(distvector), [1, ]]))
                weights = inverseGammas * gg1
                if not 0.9 < np.sum(weights[:-1]) < 1.1:
                    print self.Doy, self.VarMod, self.Layer, self.Model
                    raise AssertionError("Sum of the weights is != 1")
                estimatedValue = float(self.getModelValue(refStations.getLat(k), refStations.getLon(k)) + (self.DiffModelValues * weights[:-1]))
                MeasurementValue = self.Stations.getMeasurementFromID(stationIDs[k])
                residualsRefRel = ((MeasurementValue - estimatedValue) / MeasurementValue) * 100
                self.writeResiduals4Validation(residualsRefRel, stationIDs[k], self.OutputfolderLog + self.Layer + "_" + self.Model + self.VarMod + "ResidualsRefRel.dat")
                residualsRefAbs = MeasurementValue - estimatedValue
                self.writeResiduals4Validation(residualsRefAbs, stationIDs[k], self.OutputfolderLog + self.Layer + "_" + self.Model + self.VarMod + "ResidualsRefAbs.dat")

        return valMatrix, predErr

    def getModelValue(self, plat, plon):
        '''
        This function calculates the model value for NeQuick, NPDM or NPHM needed for the kriging estimation.
        :param plat: Latitude
        :param plon: Longitude
        :return: Modelvalue
        '''
        if "npdm" in self.Model or "nphm" in self.Model:
            allDoys = np.array(np.asarray(self.Doy), float)
            sameLat = np.ones(1) * plat
            sameLon = np.ones(1) * plon
            F10p7_sfu = np.asarray(self.F107)
            UT_hrs = self.Hour

            if self.Layer == "foF2" or self.Layer == "NmF2":
                modelObj = NEDM.model(sameLat, sameLon, UT_hrs, allDoys, F10p7_sfu)
                modelresult = modelObj.NPDM()
                modelresult *= (10**12)
            else:
                modelObj = NEDM.model(sameLat, sameLon, UT_hrs, allDoys, F10p7_sfu)
                modelresult = modelObj.NPHM()

            return modelresult

        elif "nequick" in self.Model:
            altlist = range(0, 600, 1)
            oneStation = {
            'height': np.asarray(altlist),
            'lat': np.ones(len(altlist)) * plat,
            'lon': np.ones(len(altlist)) * plon,
            'month':self.Month,
            'f10p7': np.asarray(self.F107),
            'utcHour': self.Hour
            }

            wrapper = NeQuick.NeQuickWrapperVectorInput()
            wrapper.calcFromModel(oneStation)
            results = wrapper.modelData.values()[0]
            if self.Layer == "NmF2":
                return np.max(results)
            else:
                maxVal = np.max(results)
                for i in range(len(results)):
                    if results[i] == maxVal:
                        return altlist[i]
        else: raise AssertionError("Something went horrible wrong!")

    def spatialDistanceVector(self, lats, lons, lat2, lon2):
        '''
        This function calculates a array of distances between the station coordinates and the coordinate of the prediction point
        See also Multivariate Geostatistics from Wackernagel Page 80 Equation 11.6 right hand site of the equation
        :param lats: list of all latitudes from every station
        :param lons: list of all longitudes from every station
        :param lat2: latitude of the prediction point
        :param lon2: longitude of the prediction point
        :return: the distance array
        '''
        distVector = []
        for i in range(len(lons)):
            distVector.append(self.calcDistSpherical(lats[i], lons[i], lat2, lon2))
        return distVector

    def getInverse(self, coords):
        '''
        This function returns the inverse of the kriging matrix.
        :param coords: list of the coords from every station (coords = [lat, lon])
        :return: inverse distance matrix
        '''

        distmatrix = self.spatialDistanceMatrix(coords, coords)
        if config.isotropyAnisotropy == "anisotropy":
            gammamatrix = self.getGammasMatrixAnisotropy(distmatrix)
        else:
            gammamatrix = self.getGammasMatrix(distmatrix)


        #Look at Page 80 in Wackernagel; added last row and column (Formula 11.6)
        m = np.ones( (gammamatrix.shape[0]+1, gammamatrix.shape[1]+1) )
        m[:-1,:-1] = gammamatrix
        m[len(m)-1][len(m)-1] = 0.
        try:
            result = np.matrix(m).I
        except:
            self.writeInLog("Singular Matrix!\nProblem could be the variogram or the measurement stations")
            raise AssertionError("Singular Matrix!\nProblem could be the variogram or the measurement stations")
        return result

    def getGammasMatrix(self, distmatrix):
        '''
        This function calculates the gamma matrix. The gammas are the result of the variogram function.
        See also Multivariate Geostatistics from Wackernagel Page 80 Equation 11.6 left hand site of the equation
        :param distmatrix: distance matrix
        :return: matrix of gammas
        '''
        if self.VarMod == "exp":
            for i in range(len(distmatrix)):
                for j in range(len(distmatrix)):
                    distmatrix[i][j] = self.exponential(distmatrix[i][j])
        elif self.VarMod == "sph":
            for i in range(len(distmatrix)):
                for j in range(len(distmatrix)):
                    distmatrix[i][j] = self.spherical(distmatrix[i][j])
        elif self.VarMod == "gau":
            for i in range(len(distmatrix)):
                for j in range(len(distmatrix)):
                    distmatrix[i][j] = self.gaussian(distmatrix[i][j])
        elif self.VarMod == "pow":
            for i in range(len(distmatrix)):
                for j in range(len(distmatrix)):
                    distmatrix[i][j] = self.power(distmatrix[i][j])
        else:
            raise AssertionError("Variogramodel not found! Choose between 'exp' for exponential, 'sph' for spherical, 'gau' for gaussian or 'pow' for power!")

        return distmatrix

    def spatialDistanceMatrix (self, coordsRow, coordsColumn):
        '''
        This function calculates the distance matrix.
        :param coordsRow: coordinates of every station
        :param coordsColumn: coordinates of every station (coordsColumn must be equal with coordsRow)
        :return: distance matrix
        '''
        distMatrix = np.zeros((len(coordsRow), len(coordsColumn)))
        for i in range(len(coordsRow)):
            for j in range(len(coordsColumn)):
                if i != j: distMatrix[i][j] = self.calcDistSpherical(coordsRow[i][0], coordsRow[i][1], coordsColumn[j][0], coordsColumn[j][1])
        return distMatrix

    def getGammasVector(self, distvector):
        '''
        This function calculates the gammas of the distance vector.
        See also Multivariate Geostatistics from Wackernagel Page 80 Equation 11.6 right hand site of the equation
        :param distvector:
        :return:
        '''
        gamma = []
        if self.VarMod == "exp":
            for i in range(len(distvector)): gamma.append([self.exponential(distvector[i])])
        elif self.VarMod == "sph":
            for i in range(len(distvector)): gamma.append([self.spherical(distvector[i])])
        elif self.VarMod == "gau":
            for i in range(len(distvector)): gamma.append([self.gaussian(distvector[i])])
        elif self.VarMod == "pow":
            for i in range(len(distvector)): gamma.append([self.power(distvector[i])])
        else:
            raise AssertionError("Variogramodel not found! Choose between 'exp' for exponential, 'sph' for spherical, 'gau' for gaussian or 'pow' for power!")

        return np.asarray(gamma)

    def calcDistSpherical(self, lat1, lon1, lat2, lon2):
        '''
        This function calculates the spherical distance between two points. Haversine formula used.
        :param lat1: latitude of origin
        :param lon1: longitude of origin
        :param lat2: latitude of destination
        :param lon2: longitude of destination
        :return: spherical distance of the two points
        '''
        if lat1 == lat2 and lon2 == lon1: return 0.0
        dlat = math.radians(lat2-lat1) / 2
        dlon = math.radians(lon2-lon1) / 2
        a = math.sin(dlat) * math.sin(dlat) + math.cos(math.radians(lat1)) * math.cos(math.radians(lat2)) * math.sin(dlon) * math.sin(dlon)
        c = 2 * math.atan2(math.sqrt(a), math.sqrt(1-a))
        return self.EarthRadius * c

    def plotKrigePrediction(self, valueToPlot, title):
        '''
        This function plots the krige results.
        :param ValueToPlot: prediction value
        :param title: name of the file
        :return:
        '''
        plt.figure()

        map = Basemap(projection='cyl', lat_0=0, lon_0=0, resolution='l', ellps="WGS84", area_thresh=1000.0, llcrnrlon=self.minLon,
                      llcrnrlat=self.minLat, urcrnrlon=self.maxLon, urcrnrlat=self.maxLat)

        map.drawcoastlines()
        map.drawcountries()
        map.drawmapboundary()
        parallels = np.array((60, 30, 0, -30, -60))
        map.drawparallels(parallels, labels=[1, 0, 0, 0], fontsize=12)
        meridians = np.array((90, 0, -90))
        map.drawmeridians(meridians, labels=[0, 0, 0, 1], fontsize=12)

        #get printable matrix
        counter = 0
        lenLatVec = int(180 / self.Resolution)
        lenLonVec = int(360 / self.Resolution)
        predictMatrix = np.empty(shape=(lenLatVec, lenLonVec))
        for i in range(0, lenLatVec):
            for j in range(0, lenLonVec):
                predictMatrix[i][j] = valueToPlot[i][j]
                counter += 1

        if self.Layer == "hmF2":
            colorbarscaling = np.asarray((150, 200, 250, 300, 350, 400, 450))
            vmin = 150
            vmax = 450
        else:
            colorbarscaling = np.asarray((0.0, 0.2, 0.4, 0.6, 0.8, 1.0, 1.2, 1.4, 1.6, 1.8, 2.0)) * 10**12
            vmin = 0
            vmax = 2.0 * 10**12


        image = map.imshow(predictMatrix, cmap="jet", origin='upper', norm=mc.Normalize(vmin=vmin, vmax=vmax))

        legend = map.colorbar(image, location='bottom', pad="10%", ticks=colorbarscaling)
        legend.set_clim(vmin=vmin, vmax=vmax)

        if self.DoValidation:
            refStations = self.Stations.getRefStations()
            lat = refStations.getAllLat()
            lon = refStations.getAllLon()
            measurement = refStations.getAllMeasurement()
            map.scatter(lon, lat, c=measurement, norm=mc.Normalize(vmin=vmin, vmax=vmax), marker=u'o', color='r', s=50, alpha=1, linewidths=2)


        map.scatter(self.Lons, self.Lats, c=self.MeasurementValues, norm=mc.Normalize(vmin=vmin, vmax=vmax), color='r', marker=u'^', s=50, alpha=1, linewidths=2)
        plt.title(title, fontsize=18)
        plt.savefig(self.Outputfolder + title + ".png")
        plt.close()

    def plotPredError(self, valueToPlot, title):

        plt.figure()

        map = Basemap(projection='cyl', lat_0=0, lon_0=0, resolution='l', ellps="WGS84", area_thresh=1000.0, llcrnrlon=self.minLon,
                      llcrnrlat=self.minLat, urcrnrlon=self.maxLon, urcrnrlat=self.maxLat)

        map.drawcoastlines()
        map.drawcountries()
        map.drawmapboundary()
        parallels = np.array((60, 30, 0, -30, -60))
        map.drawparallels(parallels, labels=[1, 0, 0, 0], fontsize=12)
        meridians = np.array((90, 0, -90))
        map.drawmeridians(meridians, labels=[0, 0, 0, 1], fontsize=12)

        #get printable matrix
        counter = 0
        lenLatVec = int(180 / self.Resolution)
        lenLonVec = int(360 / self.Resolution)
        predictMatrix = np.empty(shape=(lenLatVec, lenLonVec))
        for i in range(0, lenLatVec):
            for j in range(0, lenLonVec):
                predictMatrix[i][j] = valueToPlot[i][j]
                counter += 1

        #image = map.imshow(predictMatrix, cmap="jet", origin='upper')

        if self.Layer == "hmF2":
            colorbarscaling = np.asarray((0, 3, 6, 9, 12, 15, 18, 21, 24, 27, 30))
            vmin = 0
            vmax = 27
        else:
            colorbarscaling = np.asarray((0.0, 0.2, 0.4, 0.6, 0.8, 1.0, 1.2, 1.4, 1.6, 1.8, 2.0)) * 10**11
            vmin = 0
            vmax = 1.4 * 10**11


        image = map.imshow(predictMatrix, cmap="jet", origin='upper', norm=mc.Normalize(vmin=vmin, vmax=vmax))

        legend = map.colorbar(image, location='bottom', pad="10%")#, ticks=colorbarscaling)
        legend.set_clim(vmin=vmin, vmax=vmax)

        #minVal, maxVal = self.findMinMax(valueToPlot)
        #colorbarscaling = []
        #diff = (maxVal - minVal) / 9
        #for i in range(0, 9):
        #    colorbarscaling.append(minVal + diff * i)
        #colorbarscaling.append(maxVal)

        #map.colorbar(image, location='bottom', pad="10%", ticks=colorbarscaling)
        map.colorbar(image, location='bottom', pad="10%")

        if self.DoValidation:
            refStations = self.Stations.getRefStations()
            lat = refStations.getAllLat()
            lon = refStations.getAllLon()
            map.scatter(lon, lat, marker=u'o', color='black', s=50, alpha=1, linewidths=2)

        map.scatter(self.Lons, self.Lats, color='black', marker=u'^', s=50, alpha=1, linewidths=2)
        plt.title(title, fontsize=18)
        plt.savefig(self.Outputfolder + title + ".png")
        plt.close()

    def filterDistSemivar(self, distances, semivar):
        '''
        This function is used to remove the distances in the variogram, which are greater then 10.000km. The reason for this is,
        to get a better variogram estimation
        :param distances: list of all distances
        :param semivar: list of all semivariances
        :return: return a new list for all distances and semivariances
        '''
        filteredDist = []
        filteredSemivar = []
        for i in range(len(distances)):
            if (distances[i] < self.Filterdistance):
                filteredDist.append(distances[i])
                filteredSemivar.append(semivar[i])

        return filteredDist, filteredSemivar

    def calcAngle(self, lat1, lon1, lat2, lon2):
        '''
        This function calculates the angle between the two coordinates.
        :param lat1: latitude 1
        :param lon1: longitude 1
        :param lat2: latitude 2
        :param lon2: longitude 2
        :return: angle between (lat1/lon1) and (lat2/lon2)
        '''

        if lat1 == lat2 and lon1 == lon2:
            return 0
        lat1 = np.radians(lat1)
        lat2 = np.radians(lat2)
        lon1 = np.radians(lon1)
        lon2 = np.radians(lon2)

        distangle = np.arccos(np.sin(lat1) * np.sin(lat2) + np.cos(lat1) * np.cos(lat2) * np.cos(lon2 - lon1))

        angle = np.arccos((np.sin(lat2) - np.sin(lat1) * np.cos(distangle)) / (np.cos(lat1) * np.sin(distangle)))

        return np.rad2deg(angle)

    def leastSquareFit(self, param, semivar, distances):
        '''
        This function estimates the variogram parameter sill, range and nugget with the use of the least square method.
        :param param: initialized variogram parameter (param = [sill, range, nugget])
        :param semivar: list of semivariances
        :param distances: list of distances
        :return: estimated parameter in the structur [sill, range, nugget]
        '''
        varFuncRes = []

        self.Sill, self.Range, self.Nugget = param
        if self.Nugget < 0: self.Nugget = 0
        if self.Nugget > self.Sill: self.Nugget = self.Sill

        for i in range(len(distances)):
            if self.VarMod == "exp":
                varFuncRes.append(self.exponential(distances[i]))
            elif self.VarMod == "sph":
                varFuncRes.append(self.spherical(distances[i]))
            elif self.VarMod == "gau":
                varFuncRes.append(self.gaussian(distances[i]))
            elif self.VarMod == "pow":
                varFuncRes.append(self.power(distances[i]))
            else:
                raise AssertionError("Variogramodel not found! Choose between 'exp' for exponential, 'sph' for spherical, 'gau' for gaussian or 'pow' for power!")

        leastSq = np.asarray(semivar) - np.asarray(varFuncRes)
        return leastSq

    def plotVariogram(self, distances, semivar, varioparams, iso):
        '''
        This function plots the variogramcloud, the experimental variogram and the variogramcurve in both variograms.
        :param distances: list of distances
        :param semivar: list of semivariances
        :param varioparams: variogram parameter (param = [sill, range, nugget])
        :return:
        '''

        h = np.arange(0, np.max(distances), 0.1)
        varcurve = []
        for value in h:
            if self.VarMod == "exp": varcurve.append(self.exponential(value))
            elif self.VarMod == "sph": varcurve.append(self.spherical(value))
            elif self.VarMod == "gau": varcurve.append(self.gaussian(value))
            elif self.VarMod == "pow": varcurve.append(self.power(value))
            else:
                raise AssertionError("Variogramodel not found! Choose between 'exp' for exponential, 'sph' for spherical, 'gau' for gaussian or 'pow' for power!")

        plt.figure()
        plt.plot(h, varcurve, 'r-')
        plt.plot(distances, semivar, 'o')
        plt.title(varioparams)
        plt.savefig(self.Outputfolder + self.Layer + "_" + self.Model + "_" + self.VarMod + "_" + "Variogramcloud" + iso + ".png")
        plt.close()
        self.writeInListofResults(self.Layer + "_" + self.Model + "_" + self.VarMod + "_" + "Variogramcloud" + iso + ".png")

        lagIntervall = []
        plotIntervall = []
        lagTemp = self.Lag / 2
        i = 0
        countLag = 0
        lagIntervall.append(0)
        plotIntervall.append(0.25 * self.Lag)
        while countLag < self.Filterdistance:
            lagIntervall.append(lagTemp + self.Lag * i)
            if i != 0: plotIntervall.append(lagTemp * i * 2)
            i += 1
            countLag += self.Lag

        mean, counter = self.getMeanInLag(zip(distances, semivar), lagIntervall)
        plt.figure()
        plt.plot(h, varcurve, 'r-')
        plt.scatter(plotIntervall, mean, counter)
        plt.legend()
        plt.xlabel("distance")
        plt.ylabel("semivariance")
        plt.title(varioparams)
        plt.savefig(self.Outputfolder + self.Layer + "_" + self.Model + "_" + self.VarMod + "_" + "Experimental" + iso + ".png")
        plt.close()
        self.writeInListofResults(self.Layer + "_" + self.Model + "_" + self.VarMod + "_" + "Experimental" + iso + ".png")

    def getMeanInLag(self, distVal, lagIntervall):
        '''
        This function calculates the mean in a lag distance. If there is no value in a lag, zero will be added to the list
        :param distVal: zipped list of distances and semivariances (zip(distances, semivariances))
        :param lagIntervall: list of the lagintervalls
        :return: list of all means for every lag distance and a list, which stored the count of the values in every lag distance
        '''
        listOfMeans = []
        counter = []
        for i in range(len(lagIntervall)-1):
            listOfVal = []
            for j in range(len(distVal)):
                if lagIntervall[i] <= distVal[j][0] < lagIntervall[i+1]:
                    listOfVal.append(distVal[j][1])
            listOfMeans.append(self.calcmean(listOfVal))
            counter.append(len(listOfVal))

        return listOfMeans, counter

    def calcmean(self, list):
        '''
        This function calculates the mean of the list. If the list is empty, zero will be returned
        :param list: list of values
        :return: mean of the list, zero if empty
        '''
        if not list: return 0
        return np.mean(list)

    def gaussian(self, h):
        '''
        This function return the value for the gaussian variogram
        :param h: Distance between two stations
        :return: the variogram value for h
        '''
        return (self.Sill * (1 - np.exp((-3 * h**2) / self.Range**2)))

    def spherical(self, h):
        '''
        This function return the value for the spherical variogram
        :param h: Distance between two stations
        :return: the variogram value for h
        '''
        if h < self.Range: return (self.Sill * (1.5 * (h / self.Range) - 0.5 * ((h/self.Range)**3)))
        else: return self.Sill

    def exponential(self, h):
        '''
        This function return the value for the exponential variogram
        :param h: Distance between two stations
        :return: the variogram value for h
        '''
        return (self.Sill * (1 - np.exp((-3 * h) / self.Range)))

    def power(self, h):
        '''
        This function return the value for the power variogram
        :param h: Distance between two stations
        :return: the variogram value for h
        '''
        return (self.Sill * ((h / self.Range) ** self.Nugget))          #IT IS NOT THE NUGGET IT IS ONLY A NAME FOR ALPHA

    def writeInListofResults(self, filename):
        '''
        This function will write all result filenames into one file.
        :param message: message to write
        :return:
        '''
        fobj = open(self.Outputfolder + "ListOfResults.dat", "a")
        fobj.writelines(filename + "\n")
        fobj.close()

    def findMinMax(self, matrix):
        '''
        This functions finds the minimum and maximum value in a matrix
        :param matrix: Matrix
        :return: minumum value, maximum value
        '''
        minval = matrix[0][0]
        maxval = matrix[0][0]

        for i in range(len(matrix)):
            for j in range(len(matrix[0])):
                if minval > matrix[i][j]: minval = matrix[i][j]
                elif maxval < matrix[i][j]: maxval = matrix[i][j]
        return minval, maxval

    def manageEstimationLeastSquare(self, distances, semivar, iso):
        '''
        This function manages the calculation of the variogram parameter with the Least Square Method
        :param distances: list of all distances
        :param semivar: list of semivariances
        :param iso: identifier for anisotropy or isotropy
        :return: variogramparameter (sill, range, nugget) only to write on console
        '''

        distances = np.asarray(distances)
        semivar = np.asarray(semivar)

        sortind = np.argsort(distances)
        sortdist = distances[sortind]
        sortsvar = semivar[sortind]

        index = sortdist.searchsorted(np.linspace(0, np.max(distances), self.Lag+1))

        distances = [sortdist[index[i-1]:index[i]].mean() for i in range(1, len(index))]
        semivar = [sortsvar[index[i-1]:index[i]].mean() for i in range(1, len(index))]
        tempdis = []
        tempvar = []
        for i in range(len(distances)):
            if not math.isnan(distances[i]) and not math.isnan(semivar[i]):
                tempdis.append(distances[i])
                tempvar.append(semivar[i])

        semivar = tempvar
        distances = tempdis

        varrange = 5000
        varsill = (0.9 * max(self.DiffModelValues)) ** 2
        #varsill = 2e22
        varnugget = 0
        param = (varsill, varrange, varnugget)
        estimated_varioparam = optimize.leastsq(self.leastSquareFit, param, args=(semivar, distances), full_output=0)
        if estimated_varioparam:
            self.Sill, self.Range, self.Nugget = estimated_varioparam[0]
            self.Nugget = 0
        else: raise AssertionError("Bad fitting calculation!")
        
        if self.VarMod == "exp": modelname = "Exponential Variogram"
        elif self.VarMod == "sph": modelname = "Spherical Variogram"
        elif self.VarMod == "gau": modelname = "Gaussian Variogram"
        elif self.VarMod == "pow": modelname = "Power Variogram"
        else:
            raise AssertionError("Variogramodel not found! Choose between 'exp' for exponential, 'sph' for spherical, 'gau' for gaussian or 'pow' for Power!")

        varioparams = "SemiVariogram %s %s %s\n Sill:%.1f Range:%.1f Nugget:%.1f"%(self.Model, self.Layer, modelname, self.Sill, self.Range, self.Nugget)
        self.plotVariogram(distances, semivar, varioparams, iso)

        return estimated_varioparam

    def writeResiduals4Validation(self, residual, stationid, filename):
        '''
        This functions writes the residuals of a station in a file.
        :param residual: list of residuals for every station
        :param stationid: station id
        :param filename: Filename
        :return:
        '''
        fopen = open(filename, 'r')
        allLines = []
        for line in fopen:
            if stationid in line:
                line = line.replace("\n", "")
                line = line + ";" + str(residual) + "\n"
            allLines.append(line)
        fopen.close()

        fopen = open(filename, 'w')
        for i in range(len(allLines)):
            fopen.write(allLines[i])
        fopen.close()

    def writeInLog(self, message):
        '''
        This function writes all massages in a log file.
        :param message: message to write
        :return:
        '''
        fobj = open(self.OutputfolderLog + self.LogFileName, "a")
        fobj.writelines(message + "\n")
        fobj.close()

    #Anisotropy

    def manageEstimationAnisotropy(self, distances, semivar, angles):
        '''
        This function manages the anisotrope variogram estimation.
        :param distances: List of distances
        :param semivar: list of semivariances
        :param angles: list of angles
        :return:
        '''

        distance1 = []
        semivar1 = []
        for i in range(len(distances)):
            if distances[i] < self.Filterdistance and ((angles[i] <= 22.5 or angles[i] >= 337.5) or (157.5 <= angles[i] <= 202.5)):
                distance1.append(distances[i])
                semivar1.append(semivar[i])

        findModul = False
        try:
            imp.find_module('rpy2')
            findModul = True
        except:
            self.writeInLog("Warning: Could not find the modul rpy2. Variogram calculation will be done in Python with Least Square. Look up the manual for more information.")
            print "Warning: Could not find the modul rpy2. Variogram calculation will be done in Python with Least Square. Look up the manual for more information."
            findModul = False

        #Direction North - South
        if findModul:
            try:
                varioInR = VariogramInR.VariogramInR(self.Lag, self.Filterdistance, self.VarMod, self.Layer, self.Outputfolder, self.OutputfolderLog, self.Model)
                res1 = varioInR.manageEstimationInR(np.asarray(distance1), np.asarray(semivar1), "AnisotropyNorthSouth")
                if self.VarMod == "pow": self.writeInLog("Variogram parameter (AnisotropyNorthSouth) (" + self.VarMod + ") (b, alpha): " + str(res1[0]) + str(res1[2]))
                else: self.writeInLog("Variogram parameter (AnisotropyNorthSouth) (" + self.VarMod + ") (sill, range, nugget): " + str(res1[0]) + str(res1[1]) + str(res1[2]))
            except:
                self.writeInLog("Error message: " + sys.exc_info()[0])
                print "Error message: " + sys.exc_info()[0]
                res1 = self.manageEstimationLeastSquare(distance1, semivar1)
        ###############################################################
        #####################Least Squares#############################
        ###############################################################
        else:
            res1 = self.manageEstimationLeastSquare(distance1, semivar1)

        #Direction East - West
        distance2 = []
        semivar2 = []
        for i in range(len(distances)):
            if distances[i] < self.Filterdistance and ((67.5 <= angles[i] <= 112.5) or (247.5 <= angles[i] <= 292.5)):
                distance2.append(distances[i])
                semivar2.append(semivar[i])

        if findModul:
            try:
                varioInR = VariogramInR.VariogramInR(self.Lag, self.Filterdistance, self.VarMod, self.Layer, self.Outputfolder, self.OutputfolderLog, self.Model)
                res2 = varioInR.manageEstimationInR(np.asarray(distance2), np.asarray(semivar2), "AnisotropyEastWest")
                if self.VarMod == "pow": self.writeInLog("Variogram parameter (AnisotropyEastWest) (" + self.VarMod + ") (b, alpha): " + str(res2[0]) + str(res2[2]))
                else: self.writeInLog("Variogram parameter (AnisotropyEastWest) (" + self.VarMod + ") (sill, range, nugget): " + str(res2[0]) + str(res2[1]) + str(res2[2]))
            except:
                self.writeInLog("Error message: " + sys.exc_info()[0])
                print "Error message: " + sys.exc_info()[0]
                res2 = self.manageEstimationLeastSquare(distance2, semivar2)
        ###############################################################
        #####################Least Squares#############################
        ###############################################################
        else:
            res1 = self.manageEstimationLeastSquare(distance2, semivar2)

        self.RangeMatrix = np.array([[float(res1[1]), 0.0], [0.0, float(res2[1])]])        #Lambda

    def getGammasVectorAnisotropy(self, distvector):
        '''
        This function calculates the gammas of the distance vector.
        See also Multivariate Geostatistics from Wackernagel Page 80 Equation 11.6 right hand site of the equation
        :param distvector:
        :return:
        '''
        gamma = []
        if self.VarMod == "exp":
            for i in range(len(distvector)):
                h = np.linalg.det(self.RangeMatrix * np.asarray([[distvector[i]], [distvector[i]]]))
                gamma.append([self.exponential(h)])
        elif self.VarMod == "sph":
            for i in range(len(distvector)):
                h = np.linalg.det(self.RangeMatrix * np.asarray([[distvector[i]], [distvector[i]]]))
                gamma.append([self.spherical(h)])
        elif self.VarMod == "gau":
            for i in range(len(distvector)):
                h = np.linalg.det(self.RangeMatrix * np.asarray([[distvector[i]], [distvector[i]]]))
                gamma.append([self.gaussian(h)])
        elif self.VarMod == "pow":
            for i in range(len(distvector)):
                h = np.linalg.det(self.RangeMatrix * np.asarray([[distvector[i]], [distvector[i]]]))
                gamma.append([self.power(h)])
        else:
            raise AssertionError("Variogramodel not found! Choose between 'exp' for exponential, 'sph' for spherical, 'gau' for gaussian or 'pow' for power!")

        return np.asarray(gamma)

    def getGammasMatrixAnisotropy(self, distmatrix):
        '''
        This function calculates the gamma matrix. The gammas are the result of the variogram function.
        See also Multivariate Geostatistics from Wackernagel Page 80 Equation 11.6 left hand site of the equation
        :param distmatrix: distance matrix
        :return: matrix of gammas
        '''
        if self.VarMod == "exp":
            for i in range(len(distmatrix)):
                for j in range(len(distmatrix)):
                    h = np.linalg.det(self.RangeMatrix * np.asarray([[distmatrix[i][j]], [distmatrix[i][j]]]))
                    distmatrix[i][j] = self.exponential(h)
        elif self.VarMod == "sph":
            for i in range(len(distmatrix)):
                for j in range(len(distmatrix)):
                    h = np.linalg.det(self.RangeMatrix * np.asarray([[distmatrix[i][j]], [distmatrix[i][j]]]))
                    distmatrix[i][j] = self.spherical(h)
        elif self.VarMod == "gau":
            for i in range(len(distmatrix)):
                for j in range(len(distmatrix)):
                    h = np.linalg.det(self.RangeMatrix * np.asarray([[distmatrix[i][j]], [distmatrix[i][j]]]))
                    distmatrix[i][j] = self.gaussian(h)
        elif self.VarMod == "pow":
            for i in range(len(distmatrix)):
                for j in range(len(distmatrix)):
                    h = np.linalg.det(self.RangeMatrix * np.asarray([[distmatrix[i][j]], [distmatrix[i][j]]]))
                    distmatrix[i][j] = self.power(h)
        else:
            raise AssertionError("Variogramodel not found! Choose between 'exp' for exponential, 'sph' for spherical, 'gau' for gaussian or 'pow' for power!")

        return distmatrix