import os
import numpy as np
import matplotlib.pyplot as plt
import config as config
try:
    import rpy2.robjects as robjects
except:
    raise AssertionError("rpy2 is not installed!")
class VariogramInR (object):

    def __init__(self, lag, filterdistance, varmod, layer, outputfolder, outputfolderlog, model):
        '''
        Konstructor
        :param lag: Lag distance
        :param filterdistance: filter distance for the variogram
        :param varmod: variogram model
        :param layer: NmF2 or hmF2
        :param outputfolder: Path for the output results
        :param outputfolderlog: Path for the log files
        :param model: backgroundmodel
        :return:
        '''
        self.Layer = layer
        self.Lag = lag
        self.Filterdistance = filterdistance
        self.VarMod = varmod
        self.Outputfolder = outputfolder
        self.OutputfolderLog = outputfolderlog
        self.Model = model
        self.WithStructure = config.variogrammodelNoStructure

    def manageEstimationInR(self, distances, semivar, iso):
        '''
        This function manages the calculation of the variogram parameter with the help of the programming language R
        :param distances: list of all distances
        :param semivar: list of semivariances
        :param iso: identifier for anisotropy or isotropy
        :return: variogramparameter (sill, range, nugget) only to write on console
        '''

        distandvar = zip(distances, semivar)

        lagIntervall = []
        halflag = self.Lag / 2.0
        i = 1
        lagIntervall.append(0)
        lagIntervall.append(halflag)
        lagsize = 0
        while lagsize < self.Filterdistance:
            lagsize = halflag + i * self.Lag
            lagIntervall.append(lagsize)
            i += 1

        midLag = []
        lagsize = 0
        midLag.append(0)
        midLag.append(halflag/2)
        i = 1
        while lagsize < self.Filterdistance:
            lagsize = i * self.Lag
            midLag.append(lagsize)
            i += 1

        semivar, counterofValInLag = self.getMeanInLag(distandvar, midLag)

        counterLags = self.writeTemp4R(midLag, semivar, counterofValInLag)
        estimation = self.estimateVarInR(counterLags)
        #self.deleteExchangeFile()                     #not needed

        if self.VarMod == "exp": modelname = "Exponential Variogram"
        elif self.VarMod == "sph": modelname = "Spherical Variogram"
        elif self.VarMod == "gau": modelname = "Gaussian Variogram"
        elif self.VarMod == "pow": modelname = "Power Variogram"
        else:
            raise AssertionError("Variogramodel not found! Choose between 'exp' for exponential, 'sph' for spherical, 'gau' for gaussian or 'pow' for Power!")

        varioparams = "SemiVariogram %s %s %s\n Sill:%.1f Range:%.1f Nugget:%.1f"%(self.Model, self.Layer, modelname, self.Sill, self.Range, self.Nugget)
        self.plotVariogram(midLag[:-1], semivar, varioparams, iso)

        return estimation

    def writeTemp4R(self, lagIntervall, var, counterofValInLag):
        '''
        This function writes the temporary file, which contains the variances and distances.
        :param lagIntervall: list of lag intervalls
        :param var: list of mean variances in a lag
        :param counterofValInLag: list of count of values in a lag
        :return:
        '''
        fobj = open(self.OutputfolderLog + "Variogram.csv", "w")
        fobj.write("x;y;z\n")
        counter = 0
        for i in range(0, len(counterofValInLag)):
            if counterofValInLag[i] != 0 or i == 0:
                fobj.write(str(lagIntervall[i]) + ";" + "%.2f" % var[i] + ";" + str(counterofValInLag[i]) + "\n")
                counter += 1
        fobj.close()

        return counter

    def deleteExchangeFile(self):
        '''
        This function removes the exchangefile, which contains the variances and distances
        :return:
        '''
        os.remove(self.OutputfolderLog + "Exchange.csv")

    def estimateVarInR(self, amountOfLags):
        '''
        This function calculates the Sill, Range and Nugget in the programming language R.
        :param amountOfLags:
        :return:
        '''
        robjects.r.library('Rcpp')
        robjects.r.library('RGeostats')

        robjects.r('''

        main <- function(varTyp, slag, countlag, outputfolder, layer, withStructure) {
            if (varTyp == "sph") {variotyp <- 3}
            if (varTyp == "exp") {variotyp <- 2}
            if (varTyp == "gau") {variotyp <- 4}
            if (varTyp == "pow") {variotyp <- 13}

            exchangefile = paste(sep="", outputfolder, "Variogram.csv")

            tecData.table <- read.table(exchangefile, header=TRUE, sep=";")
            lags = tecData.table[1]
            var = tecData.table[2]
            count = tecData.table[3]
            db1 <- db.create(lags, var, count)
            tec.vario <- vario.calc(db1, lag=slag, nlag=countlag)
            tec.vario[1]$hh = tecData.table$x       #lag distances
            tec.vario[1]$gg = tecData.table$y       #variance
            tec.vario[1]$sw = tecData.table$z       #number of points in lag

            if (withStructure == 1)
            {
                if (layer == "NmF2")
                {
                    tec.model=model.auto(tec.vario, verbose=-1, draw=FALSE, delta=7e+22)
                }
                else
                {
                    tec.model=model.auto(tec.vario, verbose=-1, draw=FALSE, delta=1e+20)
                }
            }
            if (layer == "NmF2" & withStructure != 1)
            {
                if (variotyp == 13)
                {
                    tec.model=model.auto(tec.vario, param=c(1), struct=melem.name(c(variotyp)), flag.noreduce=T, verbose=-1, delta=7e+22, draw=FALSE)
                }
                else
                {
                    tec.model=model.auto(tec.vario, lower=c(3000, 3000), upper=c(NA, NA), struct=melem.name(c(variotyp)), flag.noreduce=T, verbose=-1, delta=7e+22, draw=FALSE)
                }
            }
            if (layer == "hmF2" & withStructure != 1)
            {
                if (variotyp == 13)
                {
                    tec.model=model.auto(tec.vario, param=c(1), struct=melem.name(c(variotyp)), flag.noreduce=T, verbose=-1, delta=7e+22, draw=FALSE)
                }
                else
                {
                    tec.model=model.auto(tec.vario, lower=c(0, 3000),  upper=c(NA, NA), struct=melem.name(c(variotyp)), flag.noreduce=T, verbose=-1, draw=FALSE, delta=1e+20)
                }
            }
            rangeSill <- c(tec.model[1]$sill, tec.model[1]$range, tec.model[1]$param)
        }

        ''')

        r_f = robjects.globalenv['main']
        res = r_f(self.VarMod, self.Lag, amountOfLags, self.OutputfolderLog, self.Layer, self.WithStructure)
        self.Sill = float(res[0])
        self.Range = float(res[1])
        if self.VarMod == "pow":
            if res[2] == 0: self.Nugget = 0.001
            else: self.Nugget = res[2]
        else: self.Nugget = 0

        if self.VarMod == "pow":
            if res[2] == 0: res[2] = 0.01
            return np.array([res[0], res[1], res[2]])
        else:
            return np.array([res[0], res[1], 0])

    def getMeanInLag(self, distVal, lagIntervall):
        '''
        This function calculates the mean in a lag distance. If there is no value in a lag, zero will be added to the list
        :param distVal: zipped list of distances and semivariances (zip(distances, semivariances))
        :param lagIntervall: list of the lagintervalls
        :return: list of all means for every lag distance and a list, which stored the count of the values in every lag distance
        '''
        listOfMeans = []
        counter = []
        for i in range(len(lagIntervall)-1):
            listOfVal = []
            for j in range(len(distVal)):
                if lagIntervall[i] <= distVal[j][0] < lagIntervall[i+1]:
                    listOfVal.append(distVal[j][1])
            listOfMeans.append(np.mean(listOfVal))
            counter.append(len(listOfVal))

        return listOfMeans, counter

    def plotVariogram(self, distances, semivar, varioparams, iso):
        '''
        This function plots the variogramcloud, the experimental variogram and the variogramcurve in both variograms.
        :param distances: list of distances
        :param semivar: list of semivariances
        :param varioparams: variogram parameter (param = [sill, range, nugget])
        :return:
        '''

        h = np.arange(0, np.max(distances), 0.1)
        varcurve = []
        for value in h:
            if self.VarMod == "exp": varcurve.append(self.exponential(value))
            elif self.VarMod == "sph": varcurve.append(self.spherical(value))
            elif self.VarMod == "gau": varcurve.append(self.gaussian(value))
            elif self.VarMod == "pow": varcurve.append(self.power(value))
            else:
                raise AssertionError("Variogramodel not found! Choose between 'exp' for exponential, 'sph' for spherical, 'gau' for gaussian or 'pow' for power!")

        plt.figure()
        plt.plot(h, varcurve, 'r-')
        plt.plot(distances, semivar, 'o')
        plt.title(varioparams)
        plt.savefig(self.Outputfolder + self.Layer + "_" + self.Model + "_" + self.VarMod + "_" + "Variogramcloud" + iso + ".png")
        plt.close()
        self.writeInListofResults(self.Layer + "_" + self.Model + "_" + self.VarMod + "_" + "Variogramcloud" + iso + ".png")

        lagIntervall = []
        plotIntervall = []
        lagTemp = self.Lag / 2
        i = 0
        countLag = 0
        lagIntervall.append(0)
        plotIntervall.append(0.25 * self.Lag)
        while countLag < self.Filterdistance:
            lagIntervall.append(lagTemp + self.Lag * i)
            if i != 0: plotIntervall.append(lagTemp * i * 2)
            i += 1
            countLag += self.Lag

        mean, counter = self.getMeanInLag(zip(distances, semivar), lagIntervall)
        plt.figure()
        plt.plot(h, varcurve, 'r-')
        plt.scatter(plotIntervall, mean, counter)
        plt.legend()
        plt.xlabel("distance")
        plt.ylabel("semivariance")
        plt.title(varioparams)
        plt.savefig(self.Outputfolder + self.Layer + "_" + self.Model + "_" + self.VarMod + "_" + "Experimental" + iso + ".png")
        plt.close()
        self.writeInListofResults(self.Layer + "_" + self.Model + "_" + self.VarMod + "_" + "Experimental" + iso + ".png")

    def writeInListofResults(self, filename):
        '''
        This function will write all result filenames into one file.
        :param message: message to write
        :return:
        '''
        fobj = open(self.Outputfolder + "ListOfResults.dat", "a")
        fobj.writelines(filename + "\n")
        fobj.close()

    def gaussian(self, h):
        '''
        This function return the value for the gaussian variogram
        :param h: Distance between two stations
        :return: the variogram value for h
        '''
        return (self.Sill * (1 - np.exp((-3 * h**2) / self.Range**2)))

    def spherical(self, h):
        '''
        This function return the value for the spherical variogram
        :param h: Distance between two stations
        :return: the variogram value for h
        '''
        if h < self.Range: return (self.Sill * (1.5 * (h / self.Range) - 0.5 * ((h/self.Range)**3)))
        else: return self.Sill

    def exponential(self, h):
        '''
        This function return the value for the exponential variogram
        :param h: Distance between two stations
        :return: the variogram value for h
        '''
        return (self.Sill * (1 - np.exp((-3 * h) / self.Range)))

    def power(self, h):
        '''
        This function return the value for the power variogram
        :param h: Distance between two stations
        :return: the variogram value for h
        '''
        return (self.Sill * ((h / self.Range) ** self.Nugget))          #IT IS NOT THE NUGGET IT IS ONLY A NAME FOR ALPHA