#!/usr/bin/env python
# -*- coding: UTF-8 -*-

#       ordinarykriging.py
#
#       Copyright 2010 Lionel Roubeyrie <lroubeyrie@limair.asso.fr>
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

__author__ = "Lionel Roubeyrie"
__contact__ = "lionel.roubeyrie at gmail dot com"

from bin.reconstruct.krigeModul.base import *


class Grid(BasicGrid):
    def __init__(self, *args):
        """ Constructs a grid from coordinates and attributes.

        Parameters
        ----------
        Can be :
        1/- x, y, v, e : sequences with same lengths
                    x, y : coordinates
                    v    : values
                    e    : standards errors. Can be omitted and will be filled
                           with 0.
            the first point is represented by x[0], y[0], v[0], e[0]
        2/- u : recarray, list of lists, ...
            the first point is represented by u[0] = [x[0], y[0], v[0], e[0]]

        """
        super(Grid, self).__init__(*args)
        self.infos = self.compInfos(self.grid)

    def __invGammatrix__(self, model, coords):
        """Computes the inverse gamma matrix of the grid.

        Parameters
        ----------
        model  : Model
                 the used model
        coords : ndarray
                 list of points coordinates [[x0,y0],[x1,y1],...]

        Returns
        -------
        ndarray
        inverse gamma matrix (like)

        """
        # model.func is vectorized...
        matrix = model.func(self.mySpatialDistanceMatrix1(coords, coords))

        #wrong distance, spherical needed
        #matrix = model.func(spatial.distance_matrix(coords, coords))

        #Unbiasedness constraints
        m = np.ones( (matrix.shape[0]+1, matrix.shape[1]+1) )
        m[:-1,:-1] = matrix
        m[-1:-1] = 0.
        res = np.matrix(m).I
        return res

    def predictedPoint(self, x, y, model, coords, values, invg):
        """Prediction of the Big Kriging for a point \o/

        Parameters
        ----------
        x, y : floats
               coordinates of the desired predicted point
        model : Model
                what model to use (and not your favorite color!)
        coords : ndarray
                 original grid coordinates
        values : ndarray
                 original grid values, ordered like coords
        invg : the resulting inverse gamma matrix based on model and coords

        Returns
        ----------
        array(x,y,v,e)
            x, y : coordinates of the desired predicted point
            v    : the predicted value
            e    : the standard error

        """

        #Distance matrix with spherical Distances
        dist = self.mySpatialDistanceMatrix2(coords, [[x, y]])

        #Wrong distanceMatrix. MAtrix with spherical distances needed
        #print len(spatial.distance_matrix(coords, [[x, y]]))

        gg = np.matrix( np.vstack([model.func(dist), [1,]]) )
        weights = invg*gg
        v = np.sum( values[:, np.newaxis]*np.asarray(weights[:-1]) )
        e = np.sqrt( abs(np.sum(gg.A1*weights.A1)) )
        return np.asarray([x, y, v, e])

    def predictedGrid(self, x, y, model, full=False):
        """Prediction for an entire grid of points
l
        Parameters
        ----------
        x, y : 1D array
               coordinates of the points on the predicted grid
        model : Model
                model to use
        full : boolean
               If False, the original grid will be downsampled for performances
               issue. Use True for a prediction based on the entire original
               grid (maybe very slow, like a european swallow)

        Returns
        ----------
        BasicGrid containing coordinates, predicted values and standards errors
        for each point.

        """
        if full:
            grid = self.grid
        else:
            print("Downsampling...")
            if self.grid.shape[0]>30:
                grid = randin(self.grid, 30)
            else:
                print("Not enough datas for downsampling. Continue with all datas...")
                grid = self.grid
        coords = np.asarray([grid.x, grid.y]).T
        invg = self.__invGammatrix__(model, coords)
        pg = [self.predictedPoint(x[i], y[i], model, coords, grid.v, invg) for i in range(len(x))]
        pg = BasicGrid(pg)
        return pg

    def mySpatialDistanceMatrix1 (self, coordsRow, coordsColumn):
        '''
        This function calculates a distance matrix considered on spherical distance
        :param coordsRow:
        :param coordsColumn:
        :return: Distance Matrix
        '''
        distMatrix = np.zeros((len(coordsRow), len(coordsColumn)))
        for i in range(len(coordsRow)):
            for j in range(len(coordsColumn)):
                if i != j: distMatrix[i][j] = self.calcDistSpehrical(coordsRow[i], coordsColumn[j])
        return distMatrix

    def mySpatialDistanceMatrix2 (self, x, y):
        '''
        This function calculates a distance matrix considered on spherical distance
        :param coordsRow:
        :param coordsColumn:
        :return: Distance Matrix
        '''
        distMatrix = np.zeros((len(x), len(y)))
        for i in range(len(x)):
                distMatrix[i][0] = self.calcDistSpehrical(x[i], y[0])
        return distMatrix

    def calcDistSpehrical(self, x, y):

        r = 6370     #Earthradius in km
        lat2 = np.radians(y[0])
        lon2 = np.radians(y[1])

        lat1 = np.radians(x[0])
        lon1 = np.radians(x[1])
        dlat = lat2 - lat1
        dlon = lon2 - lon1

        a = np.sin(dlat / 2) * np.sin(dlat / 2) + np.cos(lat1) * np.cos(lat2) * np.sin(dlon / 2) * np.sin(dlon / 2)
        c = 2 * np.arctan2(np.sqrt(a), np.sqrt(1-a))
        return r * c


def main():
    return 0

if __name__ == '__main__': main()
